# -*- coding: utf-8 -*-
import numpy as np
import pandas as pd
import logging
from  collections import OrderedDict

# pyCGM2
import pyCGM2
from pyCGM2.Utils import files
from pyCGM2.Tools import exportTools


from pyCGM2.Utils import utils

def renameEmgInAnalysis(analysisInstance,emgChannels, emgMuscles, emgContexts):

    i=len(emgChannels)-1
    for channelIt in reversed(emgChannels):
        newlabel = emgContexts[i][0] + emgMuscles[i]
        for keyIt in analysisInstance.emgStats.data.keys():
            context = keyIt[1]
            if channelIt in keyIt[0]:
                newLabelFinal = keyIt[0].replace(channelIt,newlabel)
                analysisInstance.emgStats.data[newLabelFinal,context] = analysisInstance.emgStats.data.pop((keyIt[0],context))
                logging.debug("label [%s] replaced with [%s]"%(keyIt[0],newLabelFinal))
        i=i-1






class XlsExportDataFrameFilter(object):
    """
         Filter exporting Analysis instance in xls table
    """

    def __init__(self):

        self.dataframes =list()

    def setDataFrames(self, dataframes):

        if isinstance(dataframes,pd.core.frame.DataFrame):
            dataframes=[dataframes]

        for it in dataframes:
            self.dataframes.append(it)

    def export(self,outputName, path=None,excelFormat = "xls"):
        """

        """
        i=0
        for  dataframe in self.dataframes:
            if path == None:
                if excelFormat == "xls":
                    xlsxWriter = pd.ExcelWriter((outputName + "- dataframe.xls"),engine='xlwt')
                elif excelFormat == "xlsx":
                    xlsxWriter = pd.ExcelWriter((outputName + "- dataframe.xlsx"))
            else:
                if excelFormat == "xls":
                    xlsxWriter = pd.ExcelWriter((path+outputName + "- dataframe.xls"),engine='xlwt')
                elif excelFormat == "xlsx":
                    xlsxWriter = pd.ExcelWriter((path+outputName + "- dataFrame.xlsx"))

            dataframe.to_excel(xlsxWriter,"dataframe_"+str(i),index=False)
            i+=1

        xlsxWriter.save()

class XlsAnalysisExportFilter(object):
    """
         Filter exporting Analysis instance in xls table
    """

    def __init__(self):

        self.analysis = None

    def setAnalysisInstance(self,analysisInstance):
        self.analysis = analysisInstance

    def export(self,outputName, path=None,excelFormat = "xls",mode="Advanced"):
        if mode == "Advanced":
            self.__advancedExport(outputName, path=path, excelFormat = excelFormat)
        elif mode == "Basic":
            self.__basicExport(outputName, path=path, excelFormat = excelFormat)

    def __basicExport(self,outputName, path=None,excelFormat = "xls"):
        """
            export  member *analysis* as xls file in a basic mode.
            A basic xls puts Frame number in column. Each outputs is included as new sheet.

            :Parameters:
                - `outputName` (str) - name of the xls file ( without xls extension)
                - `path` (str) - folder in which xls files will be stored
                - `excelFormat` (str) - by default : xls. xlsx is also available

        """



        if path == None:
            if excelFormat == "xls":
                xlsxWriter = pd.ExcelWriter((outputName + "- basic.xls"),engine='xlwt')
            elif excelFormat == "xlsx":
                xlsxWriter = pd.ExcelWriter((outputName + "- basic.xlsx"))
        else:
            if excelFormat == "xls":
                xlsxWriter = pd.ExcelWriter((path+outputName + "- basic.xls"),engine='xlwt')
            elif excelFormat == "xlsx":
                xlsxWriter = pd.ExcelWriter((path+outputName + "- basic.xlsx"))

        # metadata
        #--------------
        if self.analysis.subjectInfo is not  None:
            subjInfo =  self.analysis.subjectInfo
        else:
            subjInfo=None

        if self.analysis.modelInfo is not  None:
            modelInfo =  self.analysis.modelInfo
        else:
            modelInfo=None

        if self.analysis.experimentalInfo is not  None:
            experimentalConditionInfo =  self.analysis.experimentalInfo
        else:
            experimentalConditionInfo=None

        list_index =list()# use for sorting index
        if subjInfo is not None:
            for key in subjInfo:
                list_index.append(key)
            serie_subject = pd.Series(subjInfo)
        else:
            serie_subject = pd.Series()

        if modelInfo is not None:
            for key in modelInfo:
                list_index.append(key)
            serie_model = pd.Series(modelInfo)
        else:
            serie_model = pd.Series()

        if experimentalConditionInfo is not None:
            for key in experimentalConditionInfo:
                list_index.append(key)
            serie_exp = pd.Series(experimentalConditionInfo)
        else:
            serie_exp = pd.Series()

        df_metadata = pd.DataFrame({"subjectInfos": serie_subject,
                                    "modelInfos": serie_model,
                                    "experimentInfos" : serie_exp},
                                    index = list_index)


        df_metadata.to_excel(xlsxWriter,"Infos",index=False)




        if self.analysis.kinematicStats.data!={}:

            # spatio temporal paramaters matching Kinematic cycles
            dfs_l =[]
            dfs_r =[]
            for key in self.analysis.kinematicStats.pst.keys():
                label = key[0]
                context = key[1]
                n =len(self.analysis.kinematicStats.pst[label,context]['values'])
                cycle_header= ["Cycle "+str(i) for i in range(0,n)]

                if context == "Left":
                    df = pd.DataFrame.from_items([(label, self.analysis.kinematicStats.pst[label,context]['values'])],orient='index', columns=cycle_header)
                    dfs_l.append(df)

                if context == "Right":
                    df = pd.DataFrame.from_items([(label, self.analysis.kinematicStats.pst[label,context]['values'])],orient='index', columns=cycle_header)
                    dfs_r.append(df)

            if dfs_l !=[]:
                df_pst_L=pd.concat(dfs_l)
                df_pst_L.to_excel(xlsxWriter,"Left - stp-kinematics",index=False)

            if dfs_r !=[]:
                df_pst_R=pd.concat(dfs_r)
                df_pst_R.to_excel(xlsxWriter,"Right - stp-kinematics",index=False)


            # kinematic cycles
            for key in self.analysis.kinematicStats.data.keys():
                label = key[0]
                context = key[1]

                X=[]
                Y=[]
                Z=[]

                countCycle = 0
                for cycleValuesIt  in self.analysis.kinematicStats.data[label,context]["values"]:
                    if not np.all(cycleValuesIt == 0):
                        X.append(cycleValuesIt[:,0])
                        Y.append(cycleValuesIt[:,0])
                        Z.append(cycleValuesIt[:,0])
                        countCycle+=1

                X = np.asarray(X).T
                Y = np.asarray(Y).T
                Z = np.asarray(Z).T

                if X.size!=0 and Y.size!=0 and Z.size!=0:

                    cycle_header= ["Cycle "+str(i) for i in range(0,countCycle)]
                    frame_header= ["Frame "+str(i) for i in range(0,101)]


                    df_x=pd.DataFrame(X,  columns= cycle_header,index = frame_header )
                    df_x['Axis']='X'
                    df_y=pd.DataFrame(Y,  columns= cycle_header,index = frame_header )
                    df_y['Axis']='Y'
                    df_z=pd.DataFrame(Z,  columns= cycle_header,index = frame_header )
                    df_z['Axis']='Z'

                    df_label = pd.concat([df_x,df_y,df_z])
                    df_label.to_excel(xlsxWriter,str(label+"."+context),index=False)

        if self.analysis.kineticStats.data!={}:
            # spatio temporal paramaters matching Kinetic cycles
            dfs_l =[]
            dfs_r =[]
            for key in self.analysis.kineticStats.pst.keys():
                label = key[0]
                context = key[1]
                n =len(self.analysis.kineticStats.pst[label,context]['values'])
                cycle_header= ["Cycle "+str(i) for i in range(0,n)]

                if context == "Left":
                    df = pd.DataFrame.from_items([(label, self.analysis.kineticStats.pst[label,context]['values'])],orient='index', columns=cycle_header)
                    dfs_l.append(df)

                if context == "Right":
                    df = pd.DataFrame.from_items([(label, self.analysis.kineticStats.pst[label,context]['values'])],orient='index', columns=cycle_header)
                    dfs_r.append(df)

            if dfs_l !=[]:
                df_pst_L=pd.concat(dfs_l)
                df_pst_L.to_excel(xlsxWriter,"Left - pst-kinetics",index=False)

            if dfs_r !=[]:
                df_pst_R=pd.concat(dfs_r)
                df_pst_R.to_excel(xlsxWriter,"Right - pst-kinetics",index=False)


            # kinetic cycles
            for key in self.analysis.kineticStats.data.keys():
                label=key[0]
                context=key[1]

                X=[]
                Y=[]
                Z=[]

                countCycle = 0
                for cycleValuesIt  in self.analysis.kineticStats.data[label,context]["values"]:
                    if not np.all(cycleValuesIt == 0):
                        X.append(cycleValuesIt[:,0])
                        Y.append(cycleValuesIt[:,0])
                        Z.append(cycleValuesIt[:,0])
                        countCycle+=1

                X = np.asarray(X).T
                Y = np.asarray(Y).T
                Z = np.asarray(Z).T


                if X.size!=0 and Y.size!=0 and Z.size!=0:
                    cycle_header= ["Cycle "+str(i) for i in range(0,n)]
                    frame_header= ["Frame "+str(i) for i in range(0,101)]

                    df_x=pd.DataFrame(X,  columns= cycle_header,index = frame_header )
                    df_x['Axis']='X'
                    df_y=pd.DataFrame(Y,  columns= cycle_header,index = frame_header )
                    df_y['Axis']='Y'
                    df_z=pd.DataFrame(Z,  columns= cycle_header,index = frame_header )
                    df_z['Axis']='Z'

                    df_label = pd.concat([df_x,df_y,df_z])
                    df_label.to_excel(xlsxWriter,str(label+"."+context),index=False)

        xlsxWriter.save()
        logging.info("basic dataFrame [%s- basic] Exported"%outputName)





    def __advancedExport(self,outputName, path=None, excelFormat = "xls",csvFileExport =False):
        """
            export  member *analysis* as xls file in a Advanced mode.
            A Advanced xls report outputs in a single sheet and put frames in row.

            .. note::

                an advanced xls contains the folowing sheets:
                    * `descriptive stp` : descriptive statistic of spatio-tenporal parameters
                    * `stp cycles` : all cycles used for computing descriptive statistics of spatio-temporal parameters
                    * `descriptive kinematics` : descriptive statistic of kinematic parameters
                    * `kinematics cycles` : all cycles used for computing descriptive statistics of kinematic parameters
                    * `descriptive kinetics` : descriptive statistic of kinetic parameters
                    * `kinetics cycles` : all cycles used for computing descriptive statistics of kinetic parameters

            :Parameters:
                - `outputName` (str) - name of the xls file ( without xls extension)
                - `path` (str) - folder in which xls files will be stored
                - `excelFormat` (str) - by default : xls. xlsx is also available
                - `csvFileExport` (bool) - enable export of csv files

        """


        if path == None:
            if excelFormat == "xls":
                xlsxWriter = pd.ExcelWriter((outputName + "- Advanced.xls"),engine='xlwt',encoding='utf-8')
            elif excelFormat == "xlsx":
                xlsxWriter = pd.ExcelWriter((outputName + "- Advanced.xlsx"),encoding='utf-8')
        else:
            if excelFormat == "xls":
                xlsxWriter = pd.ExcelWriter((path+outputName + "- Advanced.xls"),engine='xlwt',encoding='utf-8')
            elif excelFormat == "xlsx":
                xlsxWriter = pd.ExcelWriter((path+outputName + "- Advanced.xlsx"),encoding='utf-8')

        # infos
        #-------
        if self.analysis.modelInfo is not None:
            modelInfo =  self.analysis.modelInfo
        else:
            modelInfo=None


        if self.analysis.subjectInfo is not None:
            subjInfo =  self.analysis.subjectInfo
        else:
            subjInfo=None

        if self.analysis.experimentalInfo is not None:
            condExpInfo =  self.analysis.experimentalInfo
        else:
            condExpInfo=None

        # spatio temporal parameters
        #---------------------------

        if self.analysis.stpStats != {} and self.analysis.stpStats is not None :

            # stage 1 : get descriptive data
            # --------------------------------
            df_descriptiveStp = exportTools.buid_df_descriptiveCycle1_1(self.analysis.stpStats)

            # add infos
            if modelInfo is not None:
                for key,value in modelInfo.items():
                    exportTools.isColumnNameExist( df_descriptiveStp, key)
                    df_descriptiveStp[key] = value

            if subjInfo is not None:
                for key,value in subjInfo.items():
                    exportTools.isColumnNameExist( df_descriptiveStp, key)
                    df_descriptiveStp[key] = value
            if condExpInfo is not None:
                for key,value in condExpInfo.items():
                    exportTools.isColumnNameExist( df_descriptiveStp, key)
                    df_descriptiveStp[key] = value
            df_descriptiveStp.to_excel(xlsxWriter,'descriptive stp',index=False)


            # stage 2 : get cycle values
            # --------------------------------
            df_stp = exportTools.buid_df_cycles1_1(self.analysis.stpStats)

            # add infos
            if modelInfo is not None:
                for key,value in modelInfo.items():
                    exportTools.isColumnNameExist( df_stp, key)
                    df_stp[key] = value
            if subjInfo is not None:
                for key,value in subjInfo.items():
                    exportTools.isColumnNameExist( df_stp, key)
                    df_stp[key] = value
            if condExpInfo is not None:
                for key,value in condExpInfo.items():
                    exportTools.isColumnNameExist( df_stp, key)
                    df_stp[key] = value

            df_stp.to_excel(xlsxWriter,'stp cycles',index=False)

            if csvFileExport:
                if path == None:
                    df_stp.to_csv((outputName + " - stp - DataFrame.csv"),sep=";")
                else:
                    df_stp.to_csv((path+outputName + " - stp - DataFrame.csv"),sep=";")

        # Scores
        #---------------------------

        # GPS
        if self.analysis.gps is not None:

            exportTools.buid_df_cycles1_1_onlyContext(self.analysis.gps["Context"], "Gps")


            df_descriptiveGpsByContext = exportTools.buid_df_descriptiveCycle1_1_onlyContext(self.analysis.gps["Context"], "Gps")
            df_descriptiveGpsOverall = exportTools.buid_df_descriptiveCycle1_1_overall(self.analysis.gps["Overall"],"Gps")
            df_descriptiveGps =  pd.concat([df_descriptiveGpsOverall,df_descriptiveGpsByContext])

            df_allGpsByContext = exportTools.buid_df_cycles1_1_onlyContext(self.analysis.gps["Context"], "Gps")

            # add infos
            for itdf in [df_descriptiveGps,df_descriptiveGpsByContext,df_allGpsByContext]:
                if modelInfo is not None:
                    for key,value in modelInfo.items():
                        exportTools.isColumnNameExist( itdf, key)
                        itdf[key] = value

                if subjInfo is not None:
                    for key,value in subjInfo.items():
                        exportTools.isColumnNameExist( itdf, key)
                        itdf[key] = value
                if condExpInfo is not None:
                    for key,value in condExpInfo.items():
                        exportTools.isColumnNameExist( itdf, key)
                        itdf[key] = value


            df_descriptiveGps.to_excel(xlsxWriter,'descriptive GPS',index=False)
            df_allGpsByContext.to_excel(xlsxWriter,'GPS cycles',index=False)


        if self.analysis.gvs is not None:
            df_descriptiveGvs = exportTools.buid_df_descriptiveCycle1_3(self.analysis.gvs,"Gvs")
            df_allGvs = exportTools.buid_df_cycles1_3(self.analysis.gvs, "Gvs")

            # add infos
            for itdf in [df_descriptiveGvs,df_allGvs]:
                if modelInfo is not None:
                    for key,value in modelInfo.items():
                        exportTools.isColumnNameExist( itdf, key)
                        itdf[key] = value

                if subjInfo is not None:
                    for key,value in subjInfo.items():
                        exportTools.isColumnNameExist( itdf, key)
                        itdf[key] = value
                if condExpInfo is not None:
                    for key,value in condExpInfo.items():
                        exportTools.isColumnNameExist( itdf, key)
                        itdf[key] = value


            df_descriptiveGvs.to_excel(xlsxWriter,'descriptive GVS',index=False)
            df_allGvs.to_excel(xlsxWriter,'GVS cycles',index=False)


        # Kinematics ouput
        #---------------------------


        if self.analysis.kinematicStats.data!={}:

            # stage 1 : get descriptive data
            # --------------------------------
            df_descriptiveKinematics = exportTools.buid_df_descriptiveCycle101_3(self.analysis.kinematicStats)

            # add infos
            if modelInfo is not None:
                for key,value in modelInfo.items():
                    exportTools.isColumnNameExist( df_descriptiveKinematics, key)
                    df_descriptiveKinematics[key] = value
            if subjInfo is not None:
                for key,value in subjInfo.items():
                    exportTools.isColumnNameExist( df_descriptiveKinematics, key)
                    df_descriptiveKinematics[key] = value
            if condExpInfo is not None:
                for key,value in condExpInfo.items():
                    exportTools.isColumnNameExist( df_descriptiveKinematics, key)
                    df_descriptiveKinematics[key] = value

            df_descriptiveKinematics.to_excel(xlsxWriter,'descriptive kinematics',index=False)

            # stage 2 : get cycle values
            # --------------------------------

            # cycles
            df_kinematics =  exportTools.buid_df_cycles101_3(self.analysis.kinematicStats)

            # add infos
            if modelInfo is not None:
                for key,value in modelInfo.items():
                    exportTools.isColumnNameExist( df_kinematics, key)
                    df_kinematics[key] = value

            if subjInfo is not None:
                for key,value in subjInfo.items():
                    exportTools.isColumnNameExist( df_kinematics, key)
                    df_kinematics[key] = value
            if condExpInfo is not None:
                for key,value in condExpInfo.items():
                    exportTools.isColumnNameExist( df_kinematics, key)
                    df_kinematics[key] = value

            df_kinematics.to_excel(xlsxWriter,'Kinematic cycles',index=False)
            if csvFileExport:
                if path == None:
                    df_kinematics.to_csv((outputName + " - kinematics - DataFrame.csv"),sep=";")
                else:
                    df_kinematics.to_csv((path+outputName + " - kinematics - DataFrame.csv"),sep=";")


        # Kinetic ouputs
        #---------------------------
        if self.analysis.kineticStats.data!={}:

            # stage 1 : get descriptive data
            # --------------------------------
            df_descriptiveKinetics = exportTools.buid_df_descriptiveCycle101_3(self.analysis.kineticStats)

            # add infos
            if modelInfo is not None:
                for key,value in modelInfo.items():
                    exportTools.isColumnNameExist( df_descriptiveKinetics, key)
                    df_descriptiveKinetics[key] = value
            if subjInfo is not None:
                for key,value in subjInfo.items():
                    exportTools.isColumnNameExist( df_descriptiveKinetics, key)
                    df_descriptiveKinetics[key] = value
            if condExpInfo is not None:
                for key,value in condExpInfo.items():
                    exportTools.isColumnNameExist( df_descriptiveKinetics, key)
                    df_descriptiveKinetics[key] = value

            df_descriptiveKinetics.to_excel(xlsxWriter,'descriptive kinetics',index=False)

            # stage 2 : get cycle values
            # --------------------------------

            # cycles
            df_kinetics =  exportTools.buid_df_cycles101_3(self.analysis.kineticStats)

            # add infos
            if modelInfo is not None:
                for key,value in modelInfo.items():
                    exportTools.isColumnNameExist( df_kinetics, key)
                    df_kinetics[key] = value

            if subjInfo is not None:
                for key,value in subjInfo.items():
                    exportTools.isColumnNameExist( df_kinetics, key)
                    df_kinetics[key] = value
            if condExpInfo is not None:
                for key,value in condExpInfo.items():
                    exportTools.isColumnNameExist( df_kinetics, key)
                    df_kinetics[key] = value

            df_kinetics.to_excel(xlsxWriter,'Kinetic cycles',index=False)
            if csvFileExport:
                if path == None:
                    df_kinetics.to_csv((outputName + " - kinetics - DataFrame.csv"),sep=";")
                else:
                    df_kinetics.to_csv((path+outputName + " - kinetics - DataFrame.csv"),sep=";")


        # EMG ouputs
        #---------------------------
        if self.analysis.emgStats.data!={}:

            # stage 1 : get descriptive data
            # --------------------------------
            df_descriptiveEMG = exportTools.buid_df_descriptiveCycle101_1(self.analysis.emgStats)

            # add infos
            if modelInfo is not None:
                for key,value in modelInfo.items():
                    exportTools.isColumnNameExist( df_descriptiveEMG, key)
                    df_descriptiveEMG[key] = value
            if subjInfo is not None:
                for key,value in subjInfo.items():
                    exportTools.isColumnNameExist( df_descriptiveEMG, key)
                    df_descriptiveEMG[key] = value
            if condExpInfo is not None:
                for key,value in condExpInfo.items():
                    exportTools.isColumnNameExist( df_descriptiveEMG, key)
                    df_descriptiveEMG[key] = value

            df_descriptiveEMG.to_excel(xlsxWriter,'descriptive EMG',index=False)

            # stage 2 : get cycle values
            # --------------------------------

            # cycles
            df_emg =  exportTools.buid_df_cycles101_1(self.analysis.emgStats)

            # add infos
            if modelInfo is not None:
                for key,value in modelInfo.items():
                    exportTools.isColumnNameExist( df_emg, key)
                    df_emg[key] = value

            if subjInfo is not None:
                for key,value in subjInfo.items():
                    exportTools.isColumnNameExist( df_emg, key)
                    df_emg[key] = value
            if condExpInfo is not None:
                for key,value in condExpInfo.items():
                    exportTools.isColumnNameExist( df_emg, key)
                    df_emg[key] = value

            df_emg.to_excel(xlsxWriter,'EMG cycles',index=False)
            if csvFileExport:
                if path == None:
                    df_emg.to_csv((outputName + " - EMG - DataFrame.csv"),sep=";")
                else:
                    df_emg.to_csv((path+outputName + " - EMG - DataFrame.csv"),sep=";")


        logging.info("advanced dataFrame [%s- Advanced] Exported"%outputName)

        xlsxWriter.save()


class AnalysisExportFilter(object):
    """
         Filter exporting Analysis instance in json
    """

    def __init__(self):

        self.analysis = None

    def setAnalysisInstance(self,analysisInstance):
        self.analysis = analysisInstance

    # def _build(self,data):

    def export(self,outputName, path=None):


        out=OrderedDict()

        if self.analysis.stpStats != {}:
            processedKeys=list()
            for keys in self.analysis.stpStats.keys():
                if keys not in processedKeys:
                    processedKeys.append(keys)
                else:
                    raise Exception ( "[pyCGM2] - duplicated keys[ %s - %s] detected" %(keys[0],keys[1]))

                if keys[0] not in out.keys():
                    out[keys[0]]=dict()
                    out[keys[0]][keys[1]]=dict()
                    out[keys[0]][keys[1]]["values"]=self.analysis.stpStats[keys]["values"].tolist()
                else:
                    out[keys[0]][keys[1]]=dict()
                    out[keys[0]][keys[1]]["values"]=self.analysis.stpStats[keys]["values"].tolist()

        if self.analysis.kinematicStats.data != {}:
            processedKeys=list()
            for keys in self.analysis.kinematicStats.data.keys():
                if not np.all( self.analysis.kinematicStats.data[keys]["mean"]==0):

                    if keys not in processedKeys:
                        processedKeys.append(keys)
                    else:
                        raise Exception ( "[pyCGM2] - duplicated keys[ %s - %s] detected" %(keys[0],keys[1]))

                    if keys[0] not in out.keys():
                        out[keys[0]]=dict()
                        out[keys[0]][keys[1]]=dict()
                        out[keys[0]][keys[1]]["values"]= {"X":[],"Y":[],"Z":[]}
                    else:
                        out[keys[0]][keys[1]]=dict()
                        out[keys[0]][keys[1]]["values"]= {"X":[],"Y":[],"Z":[]}

                    li_X = list()
                    for cycle in self.analysis.kinematicStats.data[keys]["values"]:
                        li_X.append(cycle[:,0].tolist())

                    li_Y = list()
                    for cycle in self.analysis.kinematicStats.data[keys]["values"]:
                        li_Y.append(cycle[:,1].tolist())

                    li_Z = list()
                    for cycle in self.analysis.kinematicStats.data[keys]["values"]:
                        li_Z.append(cycle[:,2].tolist())


                    out[keys[0]][keys[1]]["values"]["X"] = li_X
                    out[keys[0]][keys[1]]["values"]["Y"] = li_Y
                    out[keys[0]][keys[1]]["values"]["Z"] = li_Z

        if self.analysis.kineticStats.data != {}:
            processedKeys=list()
            for keys in self.analysis.kineticStats.data.keys():
                if not np.all( self.analysis.kineticStats.data[keys]["mean"]==0):

                    if keys not in processedKeys:
                        processedKeys.append(keys)
                    else:
                        raise Exception ( "[pyCGM2] - duplicated keys[ %s - %s] detected" %(keys[0],keys[1]))

                    if keys[0] not in out.keys():
                        out[keys[0]]=dict()
                        out[keys[0]][keys[1]]=dict()
                        out[keys[0]][keys[1]]["values"]= {"X":[],"Y":[],"Z":[]}
                    else:
                        out[keys[0]][keys[1]]=dict()
                        out[keys[0]][keys[1]]["values"]= {"X":[],"Y":[],"Z":[]}

                    li_X = list()
                    for cycle in self.analysis.kineticStats.data[keys]["values"]:
                        li_X.append(cycle[:,0].tolist())

                    li_Y = list()
                    for cycle in self.analysis.kineticStats.data[keys]["values"]:
                        li_Y.append(cycle[:,1].tolist())

                    li_Z = list()
                    for cycle in self.analysis.kineticStats.data[keys]["values"]:
                        li_Z.append(cycle[:,2].tolist())


                    out[keys[0]][keys[1]]["values"]["X"] = li_X
                    out[keys[0]][keys[1]]["values"]["Y"] = li_Y
                    out[keys[0]][keys[1]]["values"]["Z"] = li_Z


        if self.analysis.emgStats.data != {}:
            processedKeys=list()
            for keys in self.analysis.emgStats.data.keys():
                if not np.all( self.analysis.emgStats.data[keys]["mean"]==0):
                    if keys not in processedKeys:
                        processedKeys.append(keys)
                    else:
                        raise Exception ( "[pyCGM2] - duplicated keys[ %s - %s] detected" %(keys[0],keys[1]))

                    if keys[0] not in out.keys():
                        out[keys[0]]=dict()
                        out[keys[0]][keys[1]]=dict()
                        out[keys[0]][keys[1]]["values"]=[]
                    else:
                        out[keys[0]][keys[1]]=dict()
                        out[keys[0]][keys[1]]["values"]=[]

                    li = list()
                    for cycle in self.analysis.emgStats.data[keys]["values"]:
                        li.append(cycle[:,0].tolist())

                    out[keys[0]][keys[1]]["values"] = li

        files.saveJson(path,outputName,out)

        return out



# class AnalysisC3dExportFilter(object):
#     """
#          Filter exporting Analysis instance in json
#     """
#
#     def __init__(self):
#
#         self.analysis = None
#
#     def setAnalysisInstance(self,analysisInstance):
#         self.analysis = analysisInstance
#
#     # def _build(self,data):
#
#     def export(self,outputName, path=None):
#
#         root = ma.Node('root')
#         trial = ma.Trial("AnalysisC3d",root)
#
#         # metadata
#         #-------------
#
#         # subject infos
#         if self.analysis.subjectInfo is not None:
#             subjInfo = self.analysis.subjectInfo
#             for item in subjInfo.items():
#                 trial.setProperty("SUBJECT_INFO:"+ utils.str(item[0]),utils.str(item[1]))
#
#         # model infos
#         if self.analysis.modelInfo is not None:
#             modelInfo =  self.analysis.modelInfo
#             for item in modelInfo.items():
#                 trial.setProperty("MODEL_INFO:"+ utils.str(item[0]),utils.str(item[1]))
#
#         # model infos
#         if self.analysis.experimentalInfo is not None:
#             experimentalConditionInfo = self.analysis.experimentalInfo
#             for item in experimentalConditionInfo.items():
#                 trial.setProperty("EXPERIMENTAL_INFO:"+ utils.str(item[0]),utils.str(item[1]))
#
#
#         #trial.setProperty('MY_GROUP:MY_PARAMETER',10.0)
#
#         # kinematic cycles
#         #------------------
#
#         # metadata
#         # for key in self.analysis.kinematicStats.data.keys():
#         #     if key[1]=="Left":
#         #         n_left_cycle = len(self.analysis.kinematicStats.data[key[0],key[1]]["values"])
#         #         trial.setProperty('PROCESSING:LeftKinematicCycleNumber',n_left_cycle)
#         #         break
#         #
#         # for key in self.analysis.kinematicStats.data.keys():
#         #     if key[1]=="Right":
#         #         n_right_cycle = len(self.analysis.kinematicStats.data[key[0],key[1]]["values"])
#         #         trial.setProperty('PROCESSING:RightKinematicCycleNumber',n_right_cycle)
#         #         break
#
#         # cycles
#         for key in self.analysis.kinematicStats.data.keys():
#             label = key[0]
#             context = key[1]
#             if not np.all( self.analysis.kinematicStats.data[label,context]["mean"]==0):
#                 cycle = 0
#                 values = np.zeros((101,4))
#                 values2 = np.zeros((101,1))
#                 for val in self.analysis.kinematicStats.data[label,context]["values"]:
#                     angle = ma.TimeSequence(utils.str(label+"."+context+"."+str(cycle)),4,101,1.0,0.0,ma.TimeSequence.Type_Angle,"deg", trial.timeSequences())
#                     values[:,0:3] = val
#                     angle.setData(values)
#                     cycle+=1
#
#         # kinetic cycles
#         #------------------
#
#         # # metadata
#         # for key in self.analysis.kineticStats.data.keys():
#         #     if key[1]=="Left":
#         #         n_left_cycle = len(self.analysis.kineticStats.data[key[0],key[1]]["values"])
#         #         trial.setProperty('PROCESSING:LeftKineticCycleNumber',n_left_cycle)
#         #         break
#         #
#         # for key in self.analysis.kineticStats.data.keys():
#         #     if key[1]=="Right":
#         #         n_right_cycle = len(self.analysis.kineticStats.data[key[0],key[1]]["values"])
#         #         trial.setProperty('PROCESSING:RightKineticCycleNumber',n_right_cycle)
#         #         break
#
#         # cycles
#         for key in self.analysis.kineticStats.data.keys():
#             label = key[0]
#             context = key[1]
#             if not np.all( self.analysis.kineticStats.data[label,context]["mean"]==0):
#                 cycle = 0
#                 values = np.zeros((101,4))
#                 for val in self.analysis.kineticStats.data[label,context]["values"]:
#                     moment = ma.TimeSequence(str(label+"."+context+"."+str(cycle)),4,101,1.0,0.0,ma.TimeSequence.Type_Moment,"N.mm", trial.timeSequences())
#                     values[:,0:3] = val
#                     moment.setData(values)
#                     cycle+=1
#
#         # for key in self.analysis.emgStats.data.keys():
#         #     label = key[0]
#         #     context = key[1]
#         #     if not np.all( self.analysis.emgStats.data[label,context]["mean"]==0):
#         #         cycle = 0
#         #         for val in self.analysis.emgStats.data[label,context]["values"]:
#         #             analog = ma.TimeSequence(str(label+"."+context+"."+str(cycle)),1,101,1.0,0.0,ma.TimeSequence.Type_Analog,"V", 1.0,0.0,[-10.0,10.0], trial.timeSequences())
#         #             analog.setData(val)
#         #             cycle+=1
#
#
#         try:
#             if path == None:
#                 ma.io.write(root,utils.str(outputName+".c3d")  )
#             else:
#                 ma.io.write(root,utils.str(path + outputName+".c3d"))
#             logging.info("Analysis c3d  [%s.c3d] Exported" %( (outputName +".c3d")) )
#         except:
#             raise Exception ("[pyCGM2] : analysis c3d doesn t export" )



# -*- coding: utf-8 -*-
import numpy as np
import logging
import matplotlib.pyplot as plt
from matplotlib.path import Path

import re



from pyCGM2 import btk

from pyCGM2.Tools import  btkTools





def appendForcePlateCornerAsMarker (btkAcq):
    """
        Add a marker at each force plate corners

        :Parameters:
           - `btkAcq` (btkAcquisition) : Btk acquisition instance from a c3d

    """


    # --- ground reaction force wrench ---
    pfe = btk.btkForcePlatformsExtractor()
    pfe.SetInput(btkAcq)
    pfc = pfe.GetOutput()
    pfc.Update()


    for i in range(0,pfc.GetItemNumber()):
        val_corner0 = pfc.GetItem(i).GetCorner(0).T * np.ones((btkAcq.GetPointFrameNumber(),3))
        btkTools.smartAppendPoint(btkAcq,"fp" + str(i) + "corner0",val_corner0, desc="forcePlate")

        val_corner1 = pfc.GetItem(i).GetCorner(1).T * np.ones((btkAcq.GetPointFrameNumber(),3))
        btkTools.smartAppendPoint(btkAcq,"fp" + str(i) + "corner1",val_corner1, desc="forcePlate")

        val_corner2 = pfc.GetItem(i).GetCorner(2).T * np.ones((btkAcq.GetPointFrameNumber(),3))
        btkTools.smartAppendPoint(btkAcq,"fp" + str(i) + "corner2", val_corner2, desc="forcePlate")

        val_corner3 = pfc.GetItem(i).GetCorner(3).T * np.ones((btkAcq.GetPointFrameNumber(),3))
        btkTools.smartAppendPoint(btkAcq,"fp" + str(i) + "corner3",val_corner3, desc="forcePlate")



        val_origin = (pfc.GetItem(i).GetCorner(0)+
                      pfc.GetItem(i).GetCorner(1)+
                      pfc.GetItem(i).GetCorner(2)+
                      pfc.GetItem(i).GetCorner(3)) /4.0
        val_origin2 = val_origin.T  * np.ones((btkAcq.GetPointFrameNumber(),3))
        btkTools.smartAppendPoint(btkAcq,"fp" + str(i) + "origin",val_origin2, desc="forcePlate")




def matchingFootSideOnForceplate (btkAcq, enableRefine=True, forceThreshold=50, left_markerLabelToe ="LTOE", left_markerLabelHeel ="LHEE",
                 right_markerLabelToe ="RTOE", right_markerLabelHeel ="RHEE",  display = False, mfpa=None):
    """
        Convenient function detecting foot in contact with a force plate

        **synopsis**

        This function firsly assign foot side to FP from minimal distance with the application point of reaction force.
        A refinement is done subsequently, it confirm if foot side is valid. A foot is invalided if :

         - FP output no data superior to the set threshold
         - Foot markers are not contain in the polygon defined by force plate corner

        :Parameters:
           - `btkAcq` (btkAcquisition) - Btk acquisition instance from a c3d
           - `left_markerLabelToe` (str) - label of the left toe marker
           - `left_markerLabelHeel` (str) - label of the left heel marker
           - `right_markerLabelToe` (str) - label of the right toe marker
           - `right_markerLabelHeel` (str) - label of the right heel marker
           - `display` (bool) - display n figures ( n depend on force plate number) presenting relative distance between mid foot and the orgin of the force plate
           - `mfpa` (string or dict) - manual force plate assigmenment from another method. Can be a string (XLRA, A stand for automatic) or a dict returing assigned foot to a Force plate ID.

    """

    appendForcePlateCornerAsMarker(btkAcq)

    ff=btkAcq.GetFirstFrame()
    lf=btkAcq.GetLastFrame()
    appf=btkAcq.GetNumberAnalogSamplePerFrame()


    # --- ground reaction force wrench ---
    pfe = btk.btkForcePlatformsExtractor()
    grwf = btk.btkGroundReactionWrenchFilter()
    pfe.SetInput(btkAcq)
    pfc = pfe.GetOutput()
    grwf.SetInput(pfc)
    grwc = grwf.GetOutput()
    grwc.Update()

    midfoot_L=(btkAcq.GetPoint(left_markerLabelToe).GetValues() + btkAcq.GetPoint(left_markerLabelHeel).GetValues())/2.0
    midfoot_R=(btkAcq.GetPoint(right_markerLabelToe).GetValues() + btkAcq.GetPoint(right_markerLabelHeel).GetValues())/2.0

    suffix=str()

    if mfpa is not None:
        try:
            pfIDS=[]
            for i in range(0,pfc.GetItemNumber()):
                pfIDS.append( re.findall( "\[(.*?)\]" ,pfc.GetItem(i).GetChannel(0).GetDescription())[0])
        except Exception:
            logging.info("[pyCGM2]: Id of Force plate not detected")
            pass

    for i in range(0,grwc.GetItemNumber()):
        pos= grwc.GetItem(i).GetPosition().GetValues()
        pos_downsample = pos[0:(lf-ff+1)*appf:appf]   # downsample

        diffL = np.linalg.norm( midfoot_L-pos_downsample,axis =1)
        diffR = np.linalg.norm( midfoot_R-pos_downsample,axis =1)

        if display:
            plt.figure()
            ax = plt.subplot(1,1,1)
            plt.title("Force plate " + str(i+1))
            ax.plot(diffL,'-r')
            ax.plot(diffR,'-b')

        if np.nanmin(diffL)<np.nanmin(diffR):
            logging.debug(" Force plate " + str(i) + " : left foot")
            suffix = suffix +  "L"
        else:
            logging.debug(" Force plate " + str(i) + " : right foot")
            suffix = suffix +  "R"

    logging.debug("Matched Force plate ===> %s", (suffix))

    if enableRefine:
        # refinement of suffix
        indexFP =0

        for letter in suffix:

            force= grwc.GetItem(indexFP).GetForce().GetValues()
            force_downsample = force[0:(lf-ff+1)*appf:appf]   # downsample


            Rz = np.abs(force_downsample[:,2])

            boolLst = Rz > forceThreshold



            enableDataFlag = False
            for it in boolLst.tolist():
                if it == True:
                    enableDataFlag=True

                    break


            if not enableDataFlag:
                logging.debug("PF #%s not activated. It provides no data superior to threshold"%(str(indexFP)) )
                li = list(suffix)
                li[indexFP]="X"
                suffix ="".join(li)

            else:

                if letter =="L":
                    hee = btkAcq.GetPoint(left_markerLabelHeel).GetValues()
                    toe = btkAcq.GetPoint(left_markerLabelToe).GetValues()
                elif letter == "R":
                    hee = btkAcq.GetPoint(right_markerLabelHeel).GetValues()
                    toe = btkAcq.GetPoint(right_markerLabelToe).GetValues()

                # polygon builder
                corner0 =  btkAcq.GetPoint("fp"+ str(indexFP)+"corner0").GetValues()[0,:]
                corner1 =  btkAcq.GetPoint("fp"+ str(indexFP)+"corner1").GetValues()[0,:]
                corner2 =  btkAcq.GetPoint("fp"+ str(indexFP)+"corner2").GetValues()[0,:]
                corner3 =  btkAcq.GetPoint("fp"+ str(indexFP)+"corner3").GetValues()[0,:]

                verts = [
                    corner0[0:2], # left, bottom
                    corner1[0:2], # left, top
                    corner2[0:2], # right, top
                    corner3[0:2], # right, bottom
                    corner0[0:2], # ignored
                    ]

                codes = [Path.MOVETO,
                         Path.LINETO,
                         Path.LINETO,
                         Path.LINETO,
                         Path.CLOSEPOLY,
                         ]

                path = Path(verts, codes)



                # check if contain both toe and hee marker
                containFlags= list()
                for i in range (0, Rz.shape[0]):
                    if boolLst[i]:
                        if path.contains_point(hee[i,0:2]) and path.contains_point(toe[i,0:2]):
                            containFlags.append(True)
                        else:
                            containFlags.append(False)



                if not all(containFlags) == True:
                    logging.debug("PF #%s not activated. While Rz superior to threshold, foot markers are not contained in force plate geometry  "%(str(indexFP)) )
                    # replace only one character
                    li = list(suffix)
                    li[indexFP]="X"
                    suffix ="".join(li)



            indexFP+=1

        # correction with manual assignement
        if mfpa is not None:
            correctedSuffix=""
            if type(mfpa) == dict:
                logging.warning("[pyCGM2] : automatic force plate assigment corrected with context associated with the device Id  ")
                i=0
                for id in pfIDS:
                    fpa = mfpa[id]
                    if fpa != "A":
                        correctedSuffix = correctedSuffix + fpa
                    else:
                        correctedSuffix = correctedSuffix + suffix[i]
                    i+=1
            else:
                logging.warning("[pyCGM2] : automatic force plate assigment corrected  ")
                if len(mfpa) < len(suffix):
                    raise Exception("[pyCGM2] number of assigned force plate inferior to the number of force plate number. Your assignment should have  %s letters at least" %(str(len(suffix))))
                else:
                    if len(mfpa) > len(suffix):
                        logging.warning("[pyCGM2]: Your manual force plate assignement mentions more force plates than the number of force plates stored in the c3d")
                    for i in range(0, len(suffix)):
                        if mfpa[i] != "A":
                            correctedSuffix = correctedSuffix + mfpa[i]
                        else:
                            correctedSuffix = correctedSuffix + suffix[i]
            return correctedSuffix
        else:
            return suffix




def addForcePlateGeneralEvents (btkAcq,mappedForcePlate ):
    """
        Add General events from force plate assignmenet
    """

    ff=btkAcq.GetFirstFrame()
    lf=btkAcq.GetLastFrame()
    pf = btkAcq.GetPointFrequency()
    appf=btkAcq.GetNumberAnalogSamplePerFrame()

     # --- ground reaction force wrench ---
    pfe = btk.btkForcePlatformsExtractor()
    grwf = btk.btkGroundReactionWrenchFilter()
    pfe.SetInput(btkAcq)
    pfc = pfe.GetOutput()
    grwf.SetInput(pfc)
    grwc = grwf.GetOutput()
    grwc.Update()

    # remove force plates events
    btkTools.clearEvents(btkAcq,["Left-FP","Right-FP"])


    # add general events
    indexFP =0
    for letter in mappedForcePlate:

        force= grwc.GetItem(indexFP).GetForce().GetValues()
        force_downsample = force[0:(lf-ff+1)*appf:appf]   # downsample


        Rz = np.abs(force_downsample[:,2])

        frameMax=  ff+np.argmax(Rz)

        if letter == "L":
            ev = btk.btkEvent('Left-FP', (frameMax-1)/pf, 'General', btk.btkEvent.Automatic, '', 'event from Force plate assignment')
            btkAcq.AppendEvent(ev)
        elif letter == "R":
            ev = btk.btkEvent('Right-FP', (frameMax-1)/pf, 'General', btk.btkEvent.Automatic, '', 'event from Force plate assignment')
            btkAcq.AppendEvent(ev)


        indexFP+=1

def correctForcePlateType5(btkAcq):

    pfe = btk.btkForcePlatformsExtractor()
    pfe.SetInput(btkAcq)
    pfc = pfe.GetOutput()
    pfc.Update()

    md_force_platform =  btkAcq.GetMetaData().FindChild(str("FORCE_PLATFORM")).value()
    md_force_platform_channels = btkAcq.GetMetaData().FindChild(str("FORCE_PLATFORM")).value().FindChild(str("CHANNEL")).value()

    md_force_platform.RemoveChild("CAL_MATRIX")
    md_force_platform.RemoveChild("MATRIX_STORE")

    md_channels=list()
    for i in btkTools.smartGetMetadata(btkAcq,'FORCE_PLATFORM',"CHANNEL"):
        md_channels.append(i)


    pfds = [pfc.GetItem(0),pfc.GetItem(1)]
    forcePlateNumber = len(pfds)


    channel_number_byFp = list()
    for i in range(0,len(pfds)):
        if pfds[i].GetType() in [1,2,4]:
            channel_number_byFp.append(6)
        if pfds[i].GetType() in [3,5]:
            channel_number_byFp.append(8)

    init=0
    channel_indexes_ofAnalog=list()
    for i in channel_number_byFp:
        channel_indexes_ofAnalog.append(md_channels[init:init+i])
        init=i


    newChannelIndexes=[]
    for i in range(0,len(pfds)):

        if pfds[i].GetType() == 5:

            numAnalogs = btkAcq.GetAnalogNumber()

            analogChannels =  np.zeros((8,btkAcq.GetAnalogFrameNumber()))
            j=0
            for index in channel_indexes_ofAnalog[i]:
                analogChannels[j,:]=btkAcq.GetAnalog(int(index)-1).GetValues().T
                md_channels.remove(index)
                j+=1

            wrench = np.dot(pfds[i].GetCalMatrix().T.reshape(8,6).T,analogChannels) # warning : storage of cal_matrix of type5 force plate is wrong in btk.

            force = wrench[0:3,:].T
            moment = wrench[3:6,:].T

            origin = pfds[i].GetOrigin()
            corners = pfds[i].GetCorners()


            btkTools.smartAppendAnalog(btkAcq,"Force.Fx"+str(i),force[:,0],desc="virtual Force plate" )
            btkTools.smartAppendAnalog(btkAcq,"Force.Fy"+str(i),force[:,1],desc="virtual Force plate" )
            btkTools.smartAppendAnalog(btkAcq,"Force.Fz"+str(i),force[:,2],desc="virtual Force plate" )

            btkTools.smartAppendAnalog(btkAcq,"Moment.Mx"+str(i),moment[:,0],desc="virtual Force plate" )
            btkTools.smartAppendAnalog(btkAcq,"Moment.My"+str(i),moment[:,1],desc="virtual Force plate" )
            btkTools.smartAppendAnalog(btkAcq,"Moment.Mz"+str(i),moment[:,2],desc="virtual Force plate" )

            new_channel_indexes_ofAnalog = range(numAnalogs,numAnalogs+6)

            numAnalogs = btkAcq.GetAnalogNumber()

            btkTools.smartSetMetadata(btkAcq,'FORCE_PLATFORM',"TYPE",i,str(2))

            newChannelIndexes = newChannelIndexes +  new_channel_indexes_ofAnalog


        else:
            newChannelIndexes = newChannelIndexes + channel_indexes_ofAnalog[i]

    md_newChannelIndexes= map(lambda x: x + 1, newChannelIndexes)
    md_force_platform_channels.SetInfo(btk.btkMetaDataInfo([6,int(forcePlateNumber)], md_newChannelIndexes))

    btkAcq.GetMetaData().FindChild(str("FORCE_PLATFORM")).value().FindChild(str("ZERO")).value().SetInfo(btk.btkMetaDataInfo(btk.btkDoubleArray(forcePlateNumber, 0)))

    return btkAcq



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

"""
"""
#import ipdb
import logging
import argparse

# pyCGM2 settings
import pyCGM2
from pyCGM2 import log; log.setLoggingLevel(logging.INFO)



def main():


    parser = argparse.ArgumentParser(description='CGM1 Calibration')
    parser.add_argument('-l','--leftFlatFoot', type=int, help='left flat foot option')
    parser.add_argument('-r','--rightFlatFoot',type=int,  help='right flat foot option')
    parser.add_argument('-hf','--headFlat',type=int,  help='head flat option')
    parser.add_argument('-md','--markerDiameter', type=float, help='marker diameter')
    parser.add_argument('-ps','--pointSuffix', type=str, help='suffix of model outputs')
    parser.add_argument('--check', action='store_true', help='force model output suffix' )
    parser.add_argument('--resetMP', action='store_true', help='reset optional mass parameters')

    args = parser.parse_args()

    print (args)


if __name__ == "__main__":

    # ---- main script -----
    main()



# -*- coding: utf-8 -*-
class ConfigManager(object):
    def __init__(self,settings):
        self._userSettings = settings
        self._internSettings = None
        self.finalSettings = None

    def getFinalSettings(self):
        return self.finalSettings

    def getInternalSettings(self):
        return self._internSettings

    def getUserSettings(self):
        return self._userSettings



# -*- coding: utf-8 -*-
"""Nexus Operation : **plotSpatioTemporalParameters**

The script displays spatio-temporal parameters (Velocity, cadence, duration of the gait phases...)

:param -ps, --pointSuffix [string]: suffix adds to the pyCGM2 nomenclature

Examples:
    In the script argument box of a python nexus operation, you can edit:

    >>>  -ps=py

.. note::
    the spatio-temporal parameters are :
        * duration
        * cadence
        * stanceDuration
        * stancePhase
        * swingDuration
        * swingPhase
        * doubleStance1
        * doubleStance2
        * simpleStance
        * strideLength
        * stepLength
        * strideWidth
        * speed

.. warning::
    the spatio-temporal parameters are not stored in the c3d file yet.



"""
import traceback
import logging
import argparse
import warnings
warnings.simplefilter(action='ignore', category=FutureWarning)

import matplotlib.pyplot as plt


# pyCGM2 settings
import pyCGM2
from pyCGM2 import log; log.setLoggingLevel(logging.INFO)

# vicon nexus
try:
    import ViconNexus
except:
    from viconnexusapi import ViconNexus

# pyCGM2 libraries
from pyCGM2.Lib import analysis
from pyCGM2.Lib import plot
from pyCGM2.Report import normativeDatasets

from pyCGM2.Nexus import  nexusTools,nexusFilters
from pyCGM2.Utils import files

def main():

    plt.close("all")

    parser = argparse.ArgumentParser(description='CGM plot stp')
    parser.add_argument('-ps','--pointSuffix', type=str, help='suffix added to pyCGM2 outputs')
    args = parser.parse_args()



    NEXUS = ViconNexus.ViconNexus()
    NEXUS_PYTHON_CONNECTED = NEXUS.Client.IsConnected()

    if NEXUS_PYTHON_CONNECTED:

        pointSuffix = args.pointSuffix
        # --------------------------INPUTS ------------------------------------
        DATA_PATH, modelledFilenameNoExt = NEXUS.GetTrialName()


        modelledFilename = modelledFilenameNoExt+".c3d"

        logging.info( "data Path: "+ DATA_PATH )
        logging.info( "file: "+ modelledFilename)

        # ----- Subject -----
        # need subject to find input files
        subjects = NEXUS.GetSubjectNames()
        subject = nexusTools.getActiveSubject(NEXUS)
        logging.info(  "Subject name : " + subject  )

        # --------------------pyCGM2 MODEL ------------------------------
        model = files.loadModel(DATA_PATH,subject)
        modelVersion = model.version

        # btkAcq builder
        nacf = nexusFilters.NexusConstructAcquisitionFilter(DATA_PATH,modelledFilenameNoExt,subject)
        acq = nacf.build()


        # --------------------------PROCESSING --------------------------------
        analysisInstance = analysis.makeAnalysis(DATA_PATH,[modelledFilename], pointLabelSuffix=pointSuffix,
                                                btkAcqs=[acq])

        plot.plot_spatioTemporal(DATA_PATH,analysisInstance,
            exportPdf=True,
            outputName=modelledFilename)


    else:
        raise Exception("NO Nexus connection. Turn on Nexus")

if __name__ == "__main__":

    main()



# -*- coding: utf-8 -*-
import numpy as np
import logging

from pyCGM2.Processing import progressionFrame

import pyCGM2.Math.normalisation  as MathNormalisation

from pyCGM2 import btk
from pyCGM2.Utils import utils
from pyCGM2.Tools import btkTools
#----module methods ------


def spatioTemporelParameter_descriptiveStats(cycles,label,context):

    """
        Compute descriptive statistics of spatio-temporal parameters from a `cycles` instance

        :Parameters:
             - `cycles` (pyCGM2.Processing.cycle.Cycles) - Cycles instance built fron CycleFilter
             - `label` (str) - spatio-temporal label
             - `context` (str) - cycle side context ( Left, Right)

        :Return:
            - `outDict` (dict)  - dictionnary with descriptive statistics ( mean, std, median).  Addictional Item *val* collects cycle values

    """

    outDict=dict()

    n=len([cycle for cycle in cycles if cycle.enableFlag and cycle.context==context])
    val = np.zeros((n))

    i=0
    for cycle in cycles:
        if cycle.enableFlag and cycle.context==context:
            val[i] = cycle.getSpatioTemporalParameter(label)
            i+=1
    outDict = {'mean':np.mean(val),'std':np.std(val),'median':np.median(val),'values': val}

    return outDict


def point_descriptiveStats(cycles,label,context):
    """
        Compute descriptive statistics of point parameters from a `cycles` instance

        :Parameters:
             - `cycles` (pyCGM2.Processing.cycle.Cycles) - Cycles instance built fron CycleFilter
             - `label` (str) - point label
             - `context` (str) - cycle side context ( Left, Right)

        :Return:
            - `outDict` (dict)  - dictionnary with descriptive statistics ( mean, std, median).  Addictional Item *values* collects cycle values

    """

    outDict=dict()

    n=len([cycle for cycle in cycles if cycle.enableFlag and cycle.context==context]) # list comprehension , get number of enabled cycle

    x=np.empty((101,n))
    y=np.empty((101,n))
    z=np.empty((101,n))

    listOfPointValues=list()

    i=0
    for cycle in cycles:
        if cycle.enableFlag and cycle.context==context:
            tmp = cycle.getPointTimeSequenceDataNormalized(label)
            x[:,i]=tmp[:,0]
            y[:,i]=tmp[:,1]
            z[:,i]=tmp[:,2]

            listOfPointValues.append(tmp)

            i+=1

    meanData=np.array(np.zeros((101,3)))
    stdData=np.array(np.zeros((101,3)))
    medianData=np.array(np.zeros((101,3)))

    if not np.all(x==0):
        x[x == 0] = np.nan
        meanData[:,0] = np.nanmean(x, axis=1)
        stdData[:,0]=np.nanstd(x,axis=1)
        medianData[:,0]=np.nanmedian(x,axis=1)

    if not np.all(y==0):
        y[y == 0] = np.nan
        meanData[:,1] = np.nanmean(y, axis=1)
        stdData[:,1]=np.nanstd(y,axis=1)
        medianData[:,1]=np.nanmedian(y,axis=1)


    if not np.all(z==0):
        z[z == 0] = np.nan
        meanData[:,2] = np.nanmean(z, axis=1)
        stdData[:,2]=np.nanstd(z,axis=1)
        medianData[:,2]=np.nanmedian(z,axis=1)


    outDict = {'mean':meanData, 'median':medianData, 'std':stdData, 'values': listOfPointValues }



    return outDict



def analog_descriptiveStats(cycles,label,context):
    """
        Compute descriptive statistics of analog parameters from a `cycles` instance

        :Parameters:
             - `cycles` (pyCGM2.Processing.cycle.Cycles) - Cycles instance built fron CycleFilter
             - `label` (str) - analog label
             - `context` (str) - cycle side context ( Left, Right)

        :Return:
            - `outDict` (dict)  - dictionnary with descriptive statistics ( mean, std, median).  Addictional Item *values* collects cycle values

    """



    outDict=dict()



    n=len([cycle for cycle in cycles if cycle.enableFlag and cycle.context==context]) # list comprehension , get number of enabled cycle

    listOfPointValues=list()
    x=np.empty((101,n))

    i=0
    for cycle in cycles:
        if cycle.enableFlag and cycle.context==context:
            tmp = cycle.getAnalogTimeSequenceDataNormalized(label)
            x[:,i]=tmp[:,0]
            listOfPointValues.append(tmp)
            i+=1


    meanData=np.array(np.zeros((101,1)))
    if not np.all(x==0):
        meanData[:,0]=np.nanmean(x,axis=1)

    stdData=np.array(np.zeros((101,1)))
    if not np.all(x==0):
        stdData[:,0]=np.nanstd(x,axis=1)

    medianData=np.array(np.zeros((101,1)))
    if not np.all(x==0):
        medianData[:,0]=np.nanmedian(x,axis=1)

    maximalValues = np.max(x,axis=0)

    outDict = {'mean':meanData, 'median':medianData, 'std':stdData, 'values': listOfPointValues, 'maxs': maximalValues}

    return outDict


def construcGaitCycle(acq):
    gaitCycles=list()

    context = "Left"
    left_fs_frames=list()
    for ev in btk.Iterate(acq.GetEvents()):
        if ev.GetContext() == context and ev.GetLabel() == "Foot Strike":
            left_fs_frames.append(ev.GetFrame())

    for i in range(0, len(left_fs_frames)-1):
        gaitCycles.append (GaitCycle(acq, left_fs_frames[i],left_fs_frames[i+1],
                                       context))

    context = "Right"
    right_fs_frames=list()
    for ev in btk.Iterate(acq.GetEvents()):
        if ev.GetContext() == context and ev.GetLabel() == "Foot Strike":
            right_fs_frames.append(ev.GetFrame())

    for i in range(0, len(right_fs_frames)-1):
        gaitCycles.append (GaitCycle(acq, right_fs_frames[i],right_fs_frames[i+1],
                                       context))

    return gaitCycles

#----module classes ------

class Cycle(object):
    """
        Cut out a acq and create a generic Cycle from specific times
    """
    #pour definir un label, il faut passer par la methode setName de l objet node



    def __init__(self,acq,startFrame,endFrame,context, enableFlag = True):
        """
        :Parameters:
             - `trial` (openma-trial) - openma from a c3d
             - `startFrame` (double) -  start time of the cycle
             - `endFrame` (double) - end time of the cycle
             - `enableFlag` (bool) - flag the Cycle in order to indicate if we can use it in a analysis process.

        .. note:

            no need Time sequence of type Analog
        """


        self.acq=acq

        self.pointfrequency = float(acq.GetPointFrequency())
        self.analogfrequency = float(acq.GetAnalogFrequency())

        #self.pointfrequency = float(btkTools.smartGetMetadata(self.acq,"POINT","RATE")[0]) #trial.findChild(ma.T_TimeSequence,"",[["type",ma.TimeSequence.Type_Marker]]).sampleRate()
        #self.analogfrequency = float(btkTools.smartGetMetadata(self.acq,"ANALOG","RATE")[0]) #trial.findChild(ma.T_TimeSequence,"",[["type",ma.TimeSequence.Type_Analog]]).sampleRate()
        self.appf =  self.analogfrequency / self.pointfrequency
        self.firstFrame = acq.GetFirstFrame()
        # try:
        #     trial.findChild(ma.T_TimeSequence,"",[["type",ma.TimeSequence.Type_Marker]]).sampleRate()
        #     self.firstFrame = int(round(trial.findChild(ma.T_TimeSequence,"",[["type",ma.TimeSequence.Type_Marker]]).startFrame() * self.pointfrequency))
        #
        # except ValueError:
        #     logging.warning("[pyCGM2] : there are no time sequence of type marker in the openmaTrial")
        #     self.firstFrame = int(round(trial.findChild(ma.T_TimeSequence,"",[["type",ma.TimeSequence.Type_Analog]]).startFrame() * self.analogfrequency))/self.appf

        self.begin =  startFrame#int(round(startFrame * self.pointfrequency) + 1)
        self.end = endFrame# int(round(endFrame * self.pointfrequency) + 1)
        self.context=context
        self.enableFlag = enableFlag

        self.discreteDataList=[]

        self.stps =dict()

        logging.debug("cycle makes from Frame %d   to  %d   (%s) " % (self.begin, self.end, self.context))



    def setEnableFlag(self,flag):
        """
        set the cycle flag

        :Parameters:
             - `flag` (bool) - boolean flag

        """
        self.enableFlag = flag

    def addDiscreteData(self,label,value,instant):
        pass #todo

    def getPointTimeSequenceData(self,pointLabel):
        """
            Get temporal point data of the cycle

            :Parameters:
                - `pointLabel` (str) - point Label

        """

        if btkTools.isPointExist(self.acq,pointLabel):
            return self.acq.GetPoint(utils.str(pointLabel)).GetValues()[self.begin-self.firstFrame:self.end-self.firstFrame+1,0:3] # 0.3 because openma::Ts includes a forth column (i.e residual)
        else:
            logging.debug("[pyCGM2] the point Label %s doesn t exist " % (pointLabel))
            return None
            #raise Exception("[pyCGM2] marker %s doesn t exist"% pointLabel )


    def getPointTimeSequenceDataNormalized(self,pointLabel):
        """
            Normalisation of a point label

            :Parameters:
                - `pointLabel` (str) - point Label

        """

        data = self.getPointTimeSequenceData(pointLabel)
        if data is None:
            out=np.zeros((101,3))
        else:
            out = MathNormalisation.timeSequenceNormalisation(101,data)

        return out

    def getAnalogTimeSequenceData(self,analogLabel):
        """
            Get analog data of the cycle

            :Parameters:
                - `analogLabel` (str) - analog Label

        """
        if btkTools.isAnalogExist(self.acq,analogLabel):
            return  self.acq.GetAnalog(utils.str(analogLabel)).GetValues()[int((self.begin-self.firstFrame) * self.appf) : int((self.end-self.firstFrame+1) * self.appf),:]
        else:
            logging.debug("[pyCGM2] the Analog Label %s doesn t exist" % (analogLabel))
            return None


    def getAnalogTimeSequenceDataNormalized(self,analogLabel):
        """
            Get analog data of the cycle

            :Parameters:
                - `analogLabel` (str) - analog Label
        """

        data = self.getAnalogTimeSequenceData(analogLabel)
        if data is None:
            out=np.zeros((101,3))
        else:
            out = MathNormalisation.timeSequenceNormalisation(101,data)

        return  out

    def getEvents(self,context="All"):
        """
            Get all events of the cycle

            :Parameters:
                - `context` (str) - event context ( All, Left or Right)

        """
        events = list()
        evsn = self.acq.GetEvents()
        for ev in btk.Iterate(evsn):
            if context==("Left" or "Right") :
                if ev.GetContext()== context:
                    if ev.GetFrame() + 1  >self.begin and  ev.GetFrame() + 1<self.end:
                        events.append(ev)
            else:
                if ev.GetFrame() + 1  >self.begin and  ev.GetFrame() + 1<self.end:
                    events.append(ev)
        return events


class GaitCycle(Cycle):

    """
        Herited method of Cycle specifying a Gait Cycle

        .. note::

            - By default, X0 and Yo are the longitudinal and lateral global axis respectively
            - Each GaitCycle creation computes spatio-temporal parameters automatically.
            - spatio-temporal parameters are :
                "duration", "cadence",
                "stanceDuration", "stepDuration", "doubleStance1Duration",
                "doubleStance2Duration", "simpleStanceDuration", "stancePhase",
                "swingDuration", "swingPhase", "doubleStance1", "doubleStance2",
                "simpleStance", "stepPhase", "strideLength", "stepLength",
                "strideWidth", "speed"
    """


    STP_LABELS=["duration","cadence",
                "stanceDuration", "stepDuration", "doubleStance1Duration",
                "doubleStance2Duration","simpleStanceDuration","stancePhase",
                "swingDuration", "swingPhase", "doubleStance1", "doubleStance2",
                "simpleStance", "stepPhase","strideLength", "stepLength",
                "strideWidth", "speed"]


    def __init__(self,gaitAcq,startFrame,endFrame,context, enableFlag = True):
        """
        :Parameters:
             - `trial` (openma-trial) - openma from a c3d
             - `startFrame` (double) -  start time of the cycle
             - `endFrame` (double) - end time of the cycle
             - `enableFlag` (bool) - flag the Cycle in order to indicate if we can use it in a analysis process.

        """



        super(GaitCycle, self).__init__(gaitAcq,startFrame,endFrame,context, enableFlag = enableFlag)

        #ajout des oppositeFO, contraFO,oopositeFS
        evs=self.getEvents()


        if context=="Right":
            oppositeSide="Left"
        elif context=="Left":
            oppositeSide="Right"
        for ev in evs:
            if ev.GetLabel() == "Foot Off" and ev.GetContext()==oppositeSide:
                oppositeFO= ev.GetFrame()
            if ev.GetLabel() == "Foot Strike" and ev.GetContext()==oppositeSide:
                oppositeFS= ev.GetFrame()
            if ev.GetLabel() == "Foot Off" and ev.GetContext()==context:
                contraFO = ev.GetFrame()
        if oppositeFO > oppositeFS:
            raise Exception("[pyCGM2] : check your c3d - Gait event error")


        self.m_oppositeFO=oppositeFO
        self.m_oppositeFS=oppositeFS
        self.m_contraFO=contraFO
        self.m_normalizedOppositeFO=round(np.divide(float(self.m_oppositeFO - self.begin),float(self.end-self.begin))*100)
        self.m_normalizedOppositeFS=round(np.divide(float(self.m_oppositeFS - self.begin),float(self.end-self.begin))*100)
        self.m_normalizedContraFO=round(np.divide(float(self.m_contraFO - self.begin),float(self.end-self.begin))*100)

        self.__computeSpatioTemporalParameter()


    def __computeSpatioTemporalParameter(self):

        duration = np.divide((self.end-self.begin),self.pointfrequency)
        stanceDuration=np.divide(np.abs(self.m_contraFO - self.begin) , self.pointfrequency)
        swingDuration=np.divide(np.abs(self.m_contraFO - self.end) , self.pointfrequency)
        stepDuration=np.divide(np.abs(self.m_oppositeFS - self.begin) , self.pointfrequency)

        self.stps["duration"] = duration
        self.stps["cadence"]= np.divide(60.0,duration)

        self.stps["stanceDuration"] = stanceDuration
        self.stps["swingDuration"] =  swingDuration
        self.stps["stepDuration"] =  stepDuration
        self.stps["doubleStance1Duration"] =  np.divide(np.abs(self.m_oppositeFO - self.begin) , self.pointfrequency)
        self.stps["doubleStance2Duration"] =  np.divide(np.abs(self.m_contraFO - self.m_oppositeFS) , self.pointfrequency)
        self.stps["simpleStanceDuration"] =  np.divide(np.abs(self.m_oppositeFO - self.m_oppositeFS) , self.pointfrequency)


        self.stps["stancePhase"] =  round(np.divide(stanceDuration,duration)*100)
        self.stps["swingPhase"] =  round(np.divide(swingDuration,duration)*100 )
        self.stps["doubleStance1"] =  round(np.divide(np.divide(np.abs(self.m_oppositeFO - self.begin) , self.pointfrequency),duration)*100)
        self.stps["doubleStance2"] =  round(np.divide(np.divide(np.abs(self.m_contraFO - self.m_oppositeFS) , self.pointfrequency),duration)*100)
        self.stps["simpleStance"] =  round(np.divide(np.divide(np.abs(self.m_oppositeFO - self.m_oppositeFS) , self.pointfrequency),duration)*100)
        self.stps["stepPhase"] =  round(np.divide(stepDuration,duration)*100)

        if self.context == "Left":

            if btkTools.isPointExist(self.acq,"LHEE") and btkTools.isPointExist(self.acq,"RHEE") and btkTools.isPointExist(self.acq,"LTOE"):

                pfp = progressionFrame.PointProgressionFrameProcedure(marker="LHEE")
                pff = progressionFrame.ProgressionFrameFilter(self.acq,pfp)
                pff.compute()
                progressionAxis =  pff.outputs["progressionAxis"]
                forwardProgression = pff.outputs["forwardProgression"]
                globalFrame = pff.outputs["globalFrame"]


                longitudinal_axis=0  if progressionAxis =="X" else 1
                lateral_axis=1  if progressionAxis =="X" else 0


                strideLength=np.abs(self.getPointTimeSequenceData("LHEE")[self.end-self.begin,longitudinal_axis] -\
                                    self.getPointTimeSequenceData("LHEE")[0,longitudinal_axis])/1000.0
                self.stps["strideLength"] =  strideLength

                stepLength = np.abs(self.getPointTimeSequenceData("RHEE")[self.m_oppositeFS-self.begin,longitudinal_axis] -\
                                    self.getPointTimeSequenceData("LHEE")[0,longitudinal_axis])/1000.0
                self.stps["stepLength"] =  stepLength

                strideWidth = np.abs(self.getPointTimeSequenceData("LTOE")[self.end-self.begin,lateral_axis] -\
                                     self.getPointTimeSequenceData("RHEE")[0,lateral_axis])/1000.0
                self.stps["strideWidth"] =  strideWidth

                self.stps["speed"] = np.divide(strideLength,duration)


        if self.context == "Right":

            if btkTools.isPointExist(self.acq,"RHEE") and btkTools.isPointExist(self.acq,"LHEE") and btkTools.isPointExist(self.acq,"RTOE"):

                pfp = progressionFrame.PointProgressionFrameProcedure(marker="RHEE")
                pff = progressionFrame.ProgressionFrameFilter(self.acq,pfp)
                pff.compute()
                progressionAxis =  pff.outputs["progressionAxis"]
                forwardProgression = pff.outputs["forwardProgression"]
                globalFrame = pff.outputs["globalFrame"]

                longitudinal_axis=0  if progressionAxis =="X" else 1
                lateral_axis=1  if progressionAxis =="X" else 0

                strideLength=np.abs(self.getPointTimeSequenceData("RHEE")[self.end-self.begin,longitudinal_axis] -\
                                    self.getPointTimeSequenceData("RHEE")[0,longitudinal_axis])/1000.0

                strideWidth = np.abs(self.getPointTimeSequenceData("RTOE")[self.end-self.begin,lateral_axis] -\
                                 self.getPointTimeSequenceData("LHEE")[0,lateral_axis])/1000.0

                self.stps["strideLength"] =  strideLength
                self.stps["strideWidth"] =  strideWidth

                stepLength = np.abs(self.getPointTimeSequenceData("RHEE")[self.m_oppositeFS-self.begin,longitudinal_axis] -\
                                    self.getPointTimeSequenceData("LHEE")[0,longitudinal_axis])/1000.0
                self.stps["stepLength"] =  stepLength

                self.stps["speed"] = np.divide(strideLength,duration)

    def getSpatioTemporalParameter(self,label):
        """
        Return a spatio-temporal parameter

        :Parameters:
             - `label` (str) - label of the desired spatio-temporal parameter
        """

        return self.stps[label]


# ----- PATTERN BUILDER -----

# ----- FILTER -----
class CyclesFilter:
    """
        Filter buiding a Cycles instance.
    """

    __builder = None

    def setBuilder(self, builder):
        """
            Set the cycle builder

            :Parameters:
                 - `builder` (concrete cycleBuilder) - a concrete cycle builder

        """
        self.__builder = builder


    def build(self):
        """
            Build a `Cycles` instance
        """
        cycles = Cycles()

        spatioTemporalElements = self.__builder.getSpatioTemporal()
        cycles.setSpatioTemporalCycles(spatioTemporalElements)

        kinematicElements = self.__builder.getKinematics()
        cycles.setKinematicCycles(kinematicElements)

        kineticElements = self.__builder.getKinetics()
        cycles.setKineticCycles(kineticElements)

        emgElements = self.__builder.getEmg()
        cycles.setEmgCycles(emgElements)

        return cycles


# --- OBJECT TO BUILD
class Cycles():
    """
    Object to build from CycleFilter.

    Cycles work as **class-container**. Its attribute member collects list of build Cycle

    Attribute :

      - `spatioTemporalCycles` - (list of Cycle) - collect list of build cycles uses for spatiotemporal analysis
      - `kinematicCycles` - (list of Cycle) - collect list of build cycles uses for kinematic analysis
      - `kineticCycles` - (list of Cycle) - collect list of build cycles uses for kinetic analysis
      - `emgCycles` - (list of Cycle) - collect list of build cycles uses for emg analysis

    """

    def __init__(self):
        self.spatioTemporalCycles = None
        self.kinematicCycles = None
        self.kineticCycles = None
        self.emgCycles = None


    def setSpatioTemporalCycles(self,spatioTemporalCycles_instance):
        self.spatioTemporalCycles = spatioTemporalCycles_instance

    def setKinematicCycles(self,kinematicCycles_instance):
        self.kinematicCycles = kinematicCycles_instance

    def setKineticCycles(self,kineticCycles_instance):
        self.kineticCycles = kineticCycles_instance

    def setEmgCycles(self,emgCycles_instance):
        self.emgCycles = emgCycles_instance


# --- BUILDER
class CyclesBuilder(object):

    def __init__(self,spatioTemporalAcqs=None,kinematicAcqs=None,kineticAcqs=None,emgAcqs=None):

        self.spatioTemporalAcqs =spatioTemporalAcqs
        self.kinematicAcqs =kinematicAcqs
        self.kineticAcqs =kineticAcqs
        self.emgAcqs =emgAcqs

    def getSpatioTemporal(self):

        if self.spatioTemporalAcqs is not None:
            spatioTemporalCycles=list()
            for acq in  self.spatioTemporalAcqs:

                startFrame = acq.GetFirstFrame()
                endFrame = acq.GetLastFrame()

                context = "Left"
                left_fs_frames=list()
                for ev in btk.Iterate(acq.GetEvents()):
                    if ev.GetContext() == context and ev.GetLabel() == "Foot Strike":
                        left_fs_frames.append(ev.GetFrame())
                #
                # for ev in  acq.findChild(ma.T_Node,"SortedEvents").findChildren(ma.T_Event,"Foot Strike",[["context","Left"]]):
                #     left_fs_frames.append(ev.time())

                context = "Right"
                right_fs_frames=list()
                for ev in btk.Iterate(acq.GetEvents()):
                    if ev.GetContext() == context and ev.GetLabel() == "Foot Strike":
                        right_fs_frames.append(ev.GetFrame())


                if left_fs_frames == [] and  right_fs_frames == []:
                    spatioTemporalCycles.append (Cycle(acq, startFrame,endFrame,"Left"))
                    spatioTemporalCycles.append (Cycle(acq, startFrame,endFrame,"Right"))
                    logging.info("[pyCGM2] left and Right context - time normalization from time boudaries")

                if len(left_fs_frames) >1:
                    for i in range(0, len(left_fs_frames)-1):
                        spatioTemporalCycles.append (Cycle(acq, left_fs_frames[i],left_fs_frames[i+1],
                                                       "Left"))
                elif len(left_fs_frames) ==1:
                    logging.warning("[pyCGM2] No left cycles, only one left foot strike detected)")
                else:
                    logging.warning("[pyCGM2] No left cycles")

                if len(right_fs_frames)>1:
                    for i in range(0, len(right_fs_frames)-1):
                        spatioTemporalCycles.append (Cycle(acq, right_fs_frames[i],right_fs_frames[i+1],
                                                       "Right"))
                elif len(right_fs_frames) ==1:
                    logging.warning("[pyCGM2] No right cycles, only one right foot strike detected)")
                else:
                    logging.warning("[pyCGM2] No Right cycles")

            return spatioTemporalCycles
        else:
            return None

    def getKinematics(self):
        if self.kinematicAcqs is not None:
            kinematicCycles=list()
            for acq in  self.kinematicAcqs:

                startFrame = acq.GetFirstFrame()
                endFrame = acq.GetLastFrame()

                context = "Left"
                left_fs_frames=list()
                for ev in btk.Iterate(acq.GetEvents()):
                    if ev.GetContext() == context and ev.GetLabel() == "Foot Strike":
                        left_fs_frames.append(ev.GetFrame())


                context = "Right"
                right_fs_frames=list()
                for ev in btk.Iterate(acq.GetEvents()):
                    if ev.GetContext() == context and ev.GetLabel() == "Foot Strike":
                        right_fs_frames.append(ev.GetFrame())

                if left_fs_frames == [] and  right_fs_frames == []:
                    kinematicCycles.append (Cycle(acq, startFrame,endFrame,"Left"))
                    kinematicCycles.append (Cycle(acq, startFrame,endFrame,"Right"))
                    logging.info("[pyCGM2] left and Right context - time normalization from time boudaries")

                if len(left_fs_frames) >1:
                    for i in range(0, len(left_fs_frames)-1):
                        kinematicCycles.append (Cycle(acq, left_fs_frames[i],left_fs_frames[i+1],
                                                       "Left"))
                elif len(left_fs_frames) ==1:
                    logging.warning("[pyCGM2] No left cycles, only one left foot strike detected)")
                else:
                    logging.warning("[pyCGM2] No left cycles")

                if len(right_fs_frames) >1:
                    for i in range(0, len(right_fs_frames)-1):
                        kinematicCycles.append (Cycle(acq, right_fs_frames[i],right_fs_frames[i+1],
                                                       "Right"))
                elif len(right_fs_frames) ==1:
                    logging.warning("[pyCGM2] No right cycles, only one right foot strike detected)")
                else:
                    logging.warning("[pyCGM2] No Right cycles")

            return kinematicCycles
        else:
            return None

    def getKinetics(self):

        if self.kineticAcqs is not None:

            detectionTimeOffset = 0.02

            kineticCycles=list()
            for acq in  self.kineticAcqs:

                flag_kinetics,frames,frames_left,frames_right = btkTools.isKineticFlag(acq)

                startFrame = acq.GetFirstFrame()
                endFrame = acq.GetLastFrame()

                if flag_kinetics:

                    context = "Left"
                    left_fs_frames=list()
                    for ev in btk.Iterate(acq.GetEvents()):
                        if ev.GetContext() == context and ev.GetLabel() == "Foot Strike":
                            left_fs_frames.append(ev.GetFrame())

                    context = "Right"
                    right_fs_frames=list()
                    for ev in btk.Iterate(acq.GetEvents()):
                        if ev.GetContext() == context and ev.GetLabel() == "Foot Strike":
                            right_fs_frames.append(ev.GetFrame())


                    if left_fs_frames == [] and right_fs_frames==[]:
                        kineticCycles.append (Cycle(acq, startFrame,endFrame,"Left"))
                        kineticCycles.append (Cycle(acq, startFrame,endFrame,"Right"))
                        logging.info("[pyCGM2] left - time normalization from time boudaries")


                    count_L=0
                    if len(left_fs_frames)>1:
                        for i in range(0, len(left_fs_frames)-1):
                            init =  left_fs_frames[i]
                            end =  left_fs_frames[i+1]

                            for frameKinetic in frames_left:

                                if frameKinetic<=end and frameKinetic>=init:
                                    logging.debug("Left kinetic cycle found from %.2f to %.2f" %(left_fs_frames[i], left_fs_frames[i+1]))
                                    kineticCycles.append (Cycle(acq, left_fs_frames[i],left_fs_frames[i+1],
                                                               "Left"))

                                    count_L+=1
                        logging.debug("%i Left Kinetic cycles available" %(count_L))
                    elif len(left_fs_frames) ==1:
                        logging.warning("[pyCGM2] No left cycles, only one left foot strike detected)")
                    else:
                        logging.warning("[pyCGM2] No left cycles")

                    count_R=0
                    if len(right_fs_frames)>1:
                        for i in range(0, len(right_fs_frames)-1):
                            init =  right_fs_frames[i]
                            end =  right_fs_frames[i+1]

                            for frameKinetic in frames_right:
                                if frameKinetic<=end and frameKinetic>=init:
                                    logging.debug("Right kinetic cycle found from %.2f to %.2f" %(right_fs_frames[i], right_fs_frames[i+1]))
                                    kineticCycles.append (Cycle(acq, right_fs_frames[i],right_fs_frames[i+1],
                                                               "Right"))
                                    count_R+=1
                        logging.debug("%i Right Kinetic cycles available" %(count_R))
                    elif len(right_fs_frames) ==1:
                        logging.warning("[pyCGM2] No right cycles, only one right foot strike detected)")
                    else:
                        logging.warning("[pyCGM2] No Right cycles")

            return kineticCycles
        else:
            return None

    def getEmg(self):
        if self.emgAcqs is not None:
            emgCycles=list()
            for acq in  self.emgAcqs:

                startFrame = acq.GetFirstFrame()
                endFrame = acq.GetLastFrame()

                context = "Left"
                left_fs_frames=list()
                for ev in btk.Iterate(acq.GetEvents()):
                    if ev.GetContext() == context and ev.GetLabel() == "Foot Strike":
                        left_fs_frames.append(ev.GetFrame())

                context = "Right"
                right_fs_frames=list()
                for ev in btk.Iterate(acq.GetEvents()):
                    if ev.GetContext() == context and ev.GetLabel() == "Foot Strike":
                        right_fs_frames.append(ev.GetFrame())


                if left_fs_frames == [] and  right_fs_frames == []:
                    emgCycles.append (Cycle(acq, startFrame,endFrame,"Left"))
                    emgCycles.append (Cycle(acq, startFrame,endFrame,"Right"))
                    logging.info("[pyCGM2] left and Right context - time normalization from time boudaries")

                if len(left_fs_frames)>1:
                    for i in range(0, len(left_fs_frames)-1):
                        emgCycles.append (Cycle(acq, left_fs_frames[i],left_fs_frames[i+1],
                                                       "Left"))
                elif len(left_fs_frames) ==1:
                    logging.warning("[pyCGM2] No left cycles, only one left foot strike detected)")
                else:
                    logging.warning("[pyCGM2] No left cycles")

                if len(right_fs_frames)>1:
                    for i in range(0, len(right_fs_frames)-1):
                        emgCycles.append (Cycle(acq, right_fs_frames[i],right_fs_frames[i+1],
                                                       "Right"))
                elif len(right_fs_frames) ==1:
                    logging.warning("[pyCGM2] No right cycles, only one right foot strike detected)")
                else:
                    logging.warning("[pyCGM2] No Right cycles")

            return emgCycles

        else:
            return None

class GaitCyclesBuilder(CyclesBuilder):
    """
        Concrete builder extracting Cycles from gait trials

        .. important:: The underlying concept is spatio-temporal parameters, kinematic outputs, kinetic ouputs or emg  could be extracted from different gait trials.

    """

    def __init__(self,spatioTemporalAcqs=None,kinematicAcqs=None,kineticAcqs=None,emgAcqs=None):
        """
            :Parameters:
                 - `spatioTemporalAcqs` (list of openma trials) - list of trials of which Cycles will be extracted for computing spatio-temporal parameters
                 - `kinematicAcqs` (list of openma trials) - list of trials of which Cycles will be extracted for computing kinematic outputs
                 - `kineticAcqs` (list of openma trials) - list of trials of which Cycles will be extracted for computing kinetic outputs
                 - `emgAcqs` (list of openma trials) - list of trials of which Cycles will be extracted for emg
                 - `longitudinal_axis` (str) - label of the  longitudinal global axis (X, Y or Z)
                 - `lateral_axis` (str) - label of the  longitudinal global axis (X, Y or Z)

        """


        super(GaitCyclesBuilder, self).__init__(
            spatioTemporalAcqs = spatioTemporalAcqs,
            kinematicAcqs=kinematicAcqs,
            kineticAcqs = kineticAcqs,
            emgAcqs = emgAcqs,
            )


    def getSpatioTemporal(self):
        """
           extract Cycles used for  spatio Temporal parameters

           :return:
               -`spatioTemporalCycles` (list of GaitCycle)
        """

        if self.spatioTemporalAcqs is not None:
            spatioTemporalCycles=list()
            for acq in  self.spatioTemporalAcqs:

                context = "Left"
                left_fs_frames=list()
                for ev in btk.Iterate(acq.GetEvents()):
                    if ev.GetContext() == context and ev.GetLabel() == "Foot Strike":
                        left_fs_frames.append(ev.GetFrame())


                for i in range(0, len(left_fs_frames)-1):
                    spatioTemporalCycles.append (GaitCycle(acq, left_fs_frames[i],left_fs_frames[i+1],
                                                   context))

                context = "Right"
                right_fs_frames=list()
                for ev in btk.Iterate(acq.GetEvents()):
                    if ev.GetContext() == context and ev.GetLabel() == "Foot Strike":
                        right_fs_frames.append(ev.GetFrame())


                for i in range(0, len(right_fs_frames)-1):
                    spatioTemporalCycles.append (GaitCycle(acq, right_fs_frames[i],right_fs_frames[i+1],
                                                   context))

            return spatioTemporalCycles
        else:
            return None

    def getKinematics(self):
        """
           extract Cycles used for kinematic outputs

           :return:
             -`kinematicCycles` (list of GaitCycle)
        """

        if self.kinematicAcqs is not None:
            kinematicCycles=list()
            for acq in  self.kinematicAcqs:

                context = "Left"
                left_fs_frames=list()
                for ev in btk.Iterate(acq.GetEvents()):
                    if ev.GetContext() == context and ev.GetLabel() == "Foot Strike":
                        left_fs_frames.append(ev.GetFrame())

                for i in range(0, len(left_fs_frames)-1):
                    kinematicCycles.append (GaitCycle(acq, left_fs_frames[i],left_fs_frames[i+1],
                                                   context))

                context = "Right"
                right_fs_frames=list()
                for ev in btk.Iterate(acq.GetEvents()):
                    if ev.GetContext() == context and ev.GetLabel() == "Foot Strike":
                        right_fs_frames.append(ev.GetFrame())



                for i in range(0, len(right_fs_frames)-1):
                    kinematicCycles.append (GaitCycle(acq, right_fs_frames[i],right_fs_frames[i+1],
                                                   context))

            return kinematicCycles
        else:
            return None

    def getKinetics(self):

        if self.kineticAcqs is not None:

            detectionTimeOffset = 0.02

            kineticCycles=list()
            for acq in  self.kineticAcqs:

                flag_kinetics,frames,frames_left,frames_right = btkTools.isKineticFlag(acq)

                if flag_kinetics:
                    context = "Left"
                    count_L=0
                    left_fs_frames=list()
                    for ev in btk.Iterate(acq.GetEvents()):
                        if ev.GetContext() == context and ev.GetLabel() == "Foot Strike":
                            left_fs_frames.append(ev.GetFrame())


                    for i in range(0, len(left_fs_frames)-1):
                        init =  left_fs_frames[i]
                        end =  left_fs_frames[i+1]

                        for frameKinetic in frames_left:

                            if frameKinetic<=end and frameKinetic>=init:
                                logging.debug("Left kinetic cycle found from %.2f to %.2f" %(left_fs_frames[i], left_fs_frames[i+1]))
                                kineticCycles.append (GaitCycle(acq, left_fs_frames[i],left_fs_frames[i+1],
                                                           context))

                                count_L+=1
                    logging.debug("%i Left Kinetic cycles available" %(count_L))



                    context = "Right"
                    count_R=0
                    right_fs_frames=list()
                    for ev in btk.Iterate(acq.GetEvents()):
                        if ev.GetContext() == context and ev.GetLabel() == "Foot Strike":
                            right_fs_frames.append(ev.GetFrame())

                    for i in range(0, len(right_fs_frames)-1):
                        init =  right_fs_frames[i]
                        end =  right_fs_frames[i+1]

                        for frameKinetic in frames_right:
                            if frameKinetic<=end and frameKinetic>=init:
                                logging.debug("Right kinetic cycle found from %.2f to %.2f" %(right_fs_frames[i], right_fs_frames[i+1]))
                                kineticCycles.append (GaitCycle(acq, right_fs_frames[i],right_fs_frames[i+1],
                                                           context))
                                count_R+=1
                    logging.debug("%i Right Kinetic cycles available" %(count_R))

            return kineticCycles
        else:
            return None

    def getEmg(self):
        """
            Extract Cycles used for emg

            :return:
                -`emgCycles` (list of GaitCycle)
        """

        if self.emgAcqs is not None:
            emgCycles=list()
            for acq in  self.emgAcqs:

                context = "Left"
                left_fs_frames=list()
                for ev in btk.Iterate(acq.GetEvents()):
                    if ev.GetContext() == context and ev.GetLabel() == "Foot Strike":
                        left_fs_frames.append(ev.GetFrame())

                for i in range(0, len(left_fs_frames)-1):
                    emgCycles.append (GaitCycle(acq, left_fs_frames[i],left_fs_frames[i+1],
                                                   context))

                context = "Right"
                right_fs_frames=list()
                for ev in btk.Iterate(acq.GetEvents()):
                    if ev.GetContext() == context and ev.GetLabel() == "Foot Strike":
                        right_fs_frames.append(ev.GetFrame())

                for i in range(0, len(right_fs_frames)-1):
                    emgCycles.append (GaitCycle(acq, right_fs_frames[i],right_fs_frames[i+1],
                                                   context))

            return emgCycles
        else:
            return None



# -*- coding: utf-8 -*-
import logging
import numpy as np
import pandas as pd
import pyCGM2

from pyCGM2.Utils import files


class NormativeData(object):

    def __init__(self,filenameNoExt,modality):


        fullJsonDict = files.openFile(pyCGM2.NORMATIVE_DATABASE_PATH,filenameNoExt+".json")
        # import ipdb; ipdb.set_trace()
        keys = fullJsonDict.keys()
        self._jsonDict = fullJsonDict[keys[0]][modality]

        self.data = dict()
        self.construct()

    def construct(self):

        for key in self._jsonDict:

            self.data[key] = dict()
            self.data[key].update({"mean":None, "sd":None})

        for key in self._jsonDict:
            valueX_m2sd = np.array(self._jsonDict[key]["X"])[:,1]
            valueX_p2sd = np.array(self._jsonDict[key]["X"])[:,2]
            valueX_mean = 0.5 * (valueX_p2sd+valueX_m2sd)
            valueX_sd = valueX_p2sd-valueX_mean


            valueY_m2sd = np.array(self._jsonDict[key]["Y"])[:,1]
            valueY_p2sd = np.array(self._jsonDict[key]["Y"])[:,2]
            valueY_mean = 0.5 * (valueY_p2sd+valueY_m2sd)
            valueY_sd = valueY_p2sd-valueY_mean

            valueZ_m2sd = np.array(self._jsonDict[key]["Z"])[:,1]
            valueZ_p2sd = np.array(self._jsonDict[key]["Z"])[:,2]
            valueZ_mean = 0.5 * (valueZ_p2sd+valueZ_m2sd)
            valueZ_sd = valueZ_p2sd-valueZ_mean

            self.data[key]["mean"] = np.array([valueX_mean,valueY_mean,valueZ_mean]).T
            self.data[key]["sd"] = np.array([valueX_sd,valueY_sd,valueZ_sd]).T




def exportJson_Schwartz2008(path,name="Schwartz2008.json"):
    """generate json from schwartz2008 dataset

    :param path: path
    :type path: string
    :param name: filename
    :type name: string
    :return: json file
    :rtype: file

    normativeDatasets.exportJson_Schwartz2008(DATA_PATH,name="test.json")

    """


    def constructJson(dataset, dataset_label, modalityLabel,data):

        dataset[dataset_label].update({modalityLabel:dict()})

        for label in data.keys():

            mean = data[label]["mean"]
            sd = data[label]["sd"]

            label = label.replace(".","")
            dataset[dataset_label][modalityLabel].update({label: {'X':None,'Y':None,'Z':None }})

            outX = list()
            for i in range(0,mean.shape[0]):
                values = [i*2, mean[i,0]-sd[i,0],mean[i,0]+sd[i,0]]
                outX.append(values)

            outY = list()
            for i in range(0,mean.shape[0]):
                values = [i*2, mean[i,1]-sd[i,1],mean[i,1]+sd[i,1]]
                outY.append(values)

            outZ = list()
            for i in range(0,mean.shape[0]):
                values = [i*2, mean[i,2]-sd[i,2],mean[i,2]+sd[i,2]]
                outZ.append(values)

            dataset[dataset_label][modalityLabel][label]["X"] = outX
            dataset[dataset_label][modalityLabel][label]["Y"] = outY
            dataset[dataset_label][modalityLabel][label]["Z"] = outZ

    dataset = {"Schwartz2008": dict()}

    nds = Schwartz2008("VerySlow")
    nds.constructNormativeData()
    constructJson(dataset,"Schwartz2008","VerySlow",nds.data)

    nds = Schwartz2008("Slow")
    nds.constructNormativeData()
    constructJson(dataset,"Schwartz2008","Slow",nds.data)

    nds = Schwartz2008("Free")
    nds.constructNormativeData()
    constructJson(dataset,"Schwartz2008","Free",nds.data)

    nds = Schwartz2008("Fast")
    nds.constructNormativeData()
    constructJson(dataset,"Schwartz2008","Fast",nds.data)

    nds = Schwartz2008("VeryFast")
    nds.constructNormativeData()
    constructJson(dataset,"Schwartz2008","VeryFast",nds.data)

    files.saveJson(path, name, dataset)


class NormalSTP(object):

    def __init__(self):
        """
        """

        self.m_filename = pyCGM2.NORMATIVE_DATABASE_PATH+"stp\\normal_stp.xlsx"
        self.data = dict()

    def constructNormativeData(self):

        """
            **Description :**  Read initial xls file and construct the member dictionnary (data)
        """

        values =pd.read_excel(self.m_filename,sheetname = "Nantes")

        for index, row in values.iterrows():
            self.data[row["Label"]]={}
            self.data[row["Label"]]["Mean"] = row["Mean"]
            self.data[row["Label"]]["Std"] = row["Std"]




class Pinzone2014(object):

    def __init__(self,centre):
        """
        **Description :** Constructor of Pinzone2014_normativeDataBases


        :Parameters:
             - `centre` (str) - two choices : CentreOne or CentreTwo

        **Usage**

        .. code:: python

            from pyCGM2.Report import normativeDatabaseProcedure
            nd = normativeDatabaseProcedure.Pinzone2014_normativeDataBases("CentreOne")
            nd.constructNormativeData() # this function
            nd.data # dictionnary with all parameters extracted from the dataset CentreOne reference in Pinzone2014


        """

        self.m_filename = pyCGM2.NORMATIVE_DATABASE_PATH+"Pinzone 2014\\Formatted- Pinzone2014.xlsx"
        self.m_centre = centre
        self.data = dict()

    def __setDict(self,dataframe,JointLabel,axisLabel, dataType):
        """ populate an item of the member dictionnary (data)

        """

        if self.m_centre == "CentreOne":
            meanLabel = "CentreOneAverage"
            sdLabel = "CentreOneSD"

        elif self.m_centre == "CentreTwo":
            meanLabel = "CentreTwoAverage"
            sdLabel = "CentreTwoSD"
        else:
            raise Exception("[pyCGM2] - dont find Pinzone Normative data centre")

        if dataType == "Angles":
            self.data[JointLabel]= dict()

            data_X=dataframe[(dataframe.Angle == axisLabel[0])][meanLabel].as_matrix() if axisLabel[0] is not None else np.zeros((51))
            data_Y=dataframe[(dataframe.Angle == axisLabel[1])][meanLabel].as_matrix() if axisLabel[1] is not None else np.zeros((51))
            data_Z=dataframe[(dataframe.Angle == axisLabel[2])][meanLabel].as_matrix() if axisLabel[2] is not None else np.zeros((51))
            self.data[JointLabel]["mean"]= np.array([data_X,data_Y,data_Z]).T

            data_X=dataframe[(dataframe.Angle == axisLabel[0])][sdLabel].as_matrix() if axisLabel[0] is not None else np.zeros((51))
            data_Y=dataframe[(dataframe.Angle == axisLabel[1])][sdLabel].as_matrix()  if axisLabel[1] is not None else np.zeros((51))
            data_Z=dataframe[(dataframe.Angle == axisLabel[2])][sdLabel].as_matrix()  if axisLabel[2] is not None else np.zeros((51))
            self.data[JointLabel]["sd"] = np.array([data_X,data_Y,data_Z]).T

        if dataType == "Moments":
            self.data[JointLabel]= dict()

            data_X=dataframe[(dataframe.Moment == axisLabel[0])][meanLabel].as_matrix() if axisLabel[0] is not None else np.zeros((51))
            data_Y=dataframe[(dataframe.Moment == axisLabel[1])][meanLabel].as_matrix() if axisLabel[1] is not None else np.zeros((51))
            data_Z=dataframe[(dataframe.Moment == axisLabel[2])][meanLabel].as_matrix() if axisLabel[2] is not None else np.zeros((51))
            self.data[JointLabel]["mean"]= np.array([data_X,data_Y,data_Z]).T*1000.0

            data_X=dataframe[(dataframe.Moment == axisLabel[0])][sdLabel].as_matrix() if axisLabel[0] is not None else np.zeros((51))
            data_Y=dataframe[(dataframe.Moment == axisLabel[1])][sdLabel].as_matrix()  if axisLabel[1] is not None else np.zeros((51))
            data_Z=dataframe[(dataframe.Moment == axisLabel[2])][sdLabel].as_matrix()  if axisLabel[2] is not None else np.zeros((51))
            self.data[JointLabel]["sd"] = np.array([data_X,data_Y,data_Z]).T*1000.0

        if dataType == "Powers":
            self.data[JointLabel]= dict()

            data_X=dataframe[(dataframe.Power == axisLabel[0])][meanLabel].as_matrix() if axisLabel[0] is not None else np.zeros((51))
            data_Y=dataframe[(dataframe.Power == axisLabel[1])][meanLabel].as_matrix() if axisLabel[1] is not None else np.zeros((51))
            data_Z=dataframe[(dataframe.Power == axisLabel[2])][meanLabel].as_matrix() if axisLabel[2] is not None else np.zeros((51))
            self.data[JointLabel]["mean"]= np.array([data_X,data_Y,data_Z]).T

            data_X=dataframe[(dataframe.Power == axisLabel[0])][sdLabel].as_matrix() if axisLabel[0] is not None else np.zeros((51))
            data_Y=dataframe[(dataframe.Power == axisLabel[1])][sdLabel].as_matrix()  if axisLabel[1] is not None else np.zeros((51))
            data_Z=dataframe[(dataframe.Power == axisLabel[2])][sdLabel].as_matrix()  if axisLabel[2] is not None else np.zeros((51))
            self.data[JointLabel]["sd"] = np.array([data_X,data_Y,data_Z]).T


    def constructNormativeData(self):

        """
            **Description :**  Read initial xls file and construct the member dictionnary (data)
        """

        angles =pd.read_excel(self.m_filename,sheetname = "Angles")
        moments =pd.read_excel(self.m_filename,sheetname = "Moments")
        powers =pd.read_excel(self.m_filename,sheetname = "Powers")

        self.__setDict(angles,"Pelvis.Angles",["Pelvis Ant/Pst", "Pelvic Up/Dn", "Pelvic Int/Ext" ], "Angles")
        self.__setDict(angles,"Hip.Angles",["Hip Flx/Ext", "Hip Add/Abd", "Hip Int/Ext" ], "Angles")
        self.__setDict(angles,"Knee.Angles",["Knee Flx/Ext",None, None], "Angles")
        self.__setDict(angles,"Ankle.Angles",["Ankle Dor/Pla", None, None], "Angles")
        self.__setDict(angles,"Foot.Angles",[None, None, "Foot Int/Ext"], "Angles")

        self.__setDict(moments,"Hip.Moment",["Hip Extensor Moment ", "Hip Abductor Moment ", None ], "Moments")
        self.__setDict(moments,"Knee.Moment",["Knee Extensor Moment ", "Knee Abductor Moment ", None ], "Moments")
        self.__setDict(moments,"Ankle.Moment",["Plantarflexor Moment ", None, "Ankle Rotation Moment"], "Moments")

        self.__setDict(powers,"Hip.Power",[ None, None,"Hip Power" ], "Powers")
        self.__setDict(powers,"Knee.Power",[None, None,"Knee Power"], "Powers")
        self.__setDict(powers,"Ankle.Power",[None, None,"Ankle Power"], "Powers")


class Schwartz2008(object):


    def __init__(self,speed):

        """
        **Description :** Constructor of Schwartz2008_normativeDataBases

        :Parameters:
               - `speed` (str) -  choices : VerySlow, Slow, Free, Fast, VeryFast

        **usage**

        .. code:: python

            from pyCGM2.Report import normativeDatabaseProcedure
            nd = normativeDatabaseProcedure.Schwartz2008_normativeDataBases("Free")
            nd.constructNormativeData() # this function
            nd.data # dictionnary with all parameters extracted from the dataset CentreOne reference in Pinzone2014


        """

        self.m_filename = pyCGM2.NORMATIVE_DATABASE_PATH+"Schwartz 2008\\Formatted- Schwartz2008.xlsx"

        self.m_speedModality = speed
        self.data = dict()

    def __setDict(self,dataframe,JointLabel,axisLabel, dataType):
        """
            Populate an item of the member dictionnary (data)

        """


        if self.m_speedModality == "VerySlow":
            meanLabel = "VerySlowMean"
            sdLabel = "VerySlowSd"
        elif self.m_speedModality == "Slow":
            meanLabel = "SlowMean"
            sdLabel = "SlowSd"
        elif self.m_speedModality == "Free":
            meanLabel = "FreeMean"
            sdLabel = "FreeSd"
        elif self.m_speedModality == "Fast":
            meanLabel = "FastMean"
            sdLabel = "FastSd"
        elif self.m_speedModality == "VeryFast":
            meanLabel = "VeryFastMean"
            sdLabel = "VeryFastSd"



        if dataType == "Angles":
            self.data[JointLabel]= dict()

            data_X=dataframe[(dataframe.Angle == axisLabel[0])][meanLabel].as_matrix() if axisLabel[0] is not None else np.zeros((51))
            data_Y=dataframe[(dataframe.Angle == axisLabel[1])][meanLabel].as_matrix() if axisLabel[1] is not None else np.zeros((51))
            data_Z=dataframe[(dataframe.Angle == axisLabel[2])][meanLabel].as_matrix() if axisLabel[2] is not None else np.zeros((51))
            self.data[JointLabel]["mean"]= np.array([data_X,data_Y,data_Z]).T

            data_X=dataframe[(dataframe.Angle == axisLabel[0])][sdLabel].as_matrix() if axisLabel[0] is not None else np.zeros((51))
            data_Y=dataframe[(dataframe.Angle == axisLabel[1])][sdLabel].as_matrix()  if axisLabel[1] is not None else np.zeros((51))
            data_Z=dataframe[(dataframe.Angle == axisLabel[2])][sdLabel].as_matrix()  if axisLabel[2] is not None else np.zeros((51))
            self.data[JointLabel]["sd"] = np.array([data_X,data_Y,data_Z]).T

        if dataType == "Moments":
            self.data[JointLabel]= dict()

            data_X=dataframe[(dataframe.Moment == axisLabel[0])][meanLabel].as_matrix() if axisLabel[0] is not None else np.zeros((51))
            data_Y=dataframe[(dataframe.Moment == axisLabel[1])][meanLabel].as_matrix() if axisLabel[1] is not None else np.zeros((51))
            data_Z=dataframe[(dataframe.Moment == axisLabel[2])][meanLabel].as_matrix() if axisLabel[2] is not None else np.zeros((51))
            self.data[JointLabel]["mean"]= np.array([data_X,data_Y,data_Z]).T*1000.0

            data_X=dataframe[(dataframe.Moment == axisLabel[0])][sdLabel].as_matrix() if axisLabel[0] is not None else np.zeros((51))
            data_Y=dataframe[(dataframe.Moment == axisLabel[1])][sdLabel].as_matrix()  if axisLabel[1] is not None else np.zeros((51))
            data_Z=dataframe[(dataframe.Moment == axisLabel[2])][sdLabel].as_matrix()  if axisLabel[2] is not None else np.zeros((51))
            self.data[JointLabel]["sd"] = np.array([data_X,data_Y,data_Z]).T*1000.0

        if dataType == "Powers":
            self.data[JointLabel]= dict()

            data_X=dataframe[(dataframe.Power == axisLabel[0])][meanLabel].as_matrix() if axisLabel[0] is not None else np.zeros((51))
            data_Y=dataframe[(dataframe.Power == axisLabel[1])][meanLabel].as_matrix() if axisLabel[1] is not None else np.zeros((51))
            data_Z=dataframe[(dataframe.Power == axisLabel[2])][meanLabel].as_matrix() if axisLabel[2] is not None else np.zeros((51))
            self.data[JointLabel]["mean"]= np.array([data_X,data_Y,data_Z]).T

            data_X=dataframe[(dataframe.Power == axisLabel[0])][sdLabel].as_matrix() if axisLabel[0] is not None else np.zeros((51))
            data_Y=dataframe[(dataframe.Power == axisLabel[1])][sdLabel].as_matrix()  if axisLabel[1] is not None else np.zeros((51))
            data_Z=dataframe[(dataframe.Power == axisLabel[2])][sdLabel].as_matrix()  if axisLabel[2] is not None else np.zeros((51))
            self.data[JointLabel]["sd"] = np.array([data_X,data_Y,data_Z]).T


    def constructNormativeData(self):
        """
            **Description :**  Read initial xls file and construct the member dictionnary (data)
        """

        angles =pd.read_excel(self.m_filename,sheetname = "Joint Rotations")
        moments =pd.read_excel(self.m_filename,sheetname = "Joint Moments")
        powers =pd.read_excel(self.m_filename,sheetname = "Joint Power")


        self.__setDict(angles,"Pelvis.Angles",["Pelvic Ant/Posterior Tilt", "Pelvic Up/Down Obliquity", "Pelvic Int/External Rotation" ], "Angles")
        self.__setDict(angles,"Hip.Angles",["Hip Flex/Extension", "Hip Ad/Abduction", "Hip Int/External Rotation" ],"Angles")
        self.__setDict(angles,"Knee.Angles",["Knee Flex/Extension","Knee Ad/Abduction", "Knee Int/External Rotation"], "Angles")
        self.__setDict(angles,"Ankle.Angles",["Ankle Dorsi/Plantarflexion", None, None], "Angles")
        self.__setDict(angles,"Foot.Angles",[None, None, "Foot Int/External Progression"], "Angles")

        self.__setDict(moments,"Hip.Moment",["Hip Ext/Flexion", "Hip Ab/Adduction", None ], "Moments")
        self.__setDict(moments,"Knee.Moment",["Knee Ext/Flexion", "Knee Ab/Adduction", None ], "Moments")
        self.__setDict(moments,"Ankle.Moment",["Ankle Dorsi/Plantarflexion", None, None], "Moments")

        self.__setDict(powers,"Hip.Power",[ None, None,"Hip" ], "Powers")
        self.__setDict(powers,"Knee.Power",[None, None,"Knee"], "Powers")
        self.__setDict(powers,"Ankle.Power",[None, None,"Ankle"], "Powers")



# -*- coding: utf-8 -*-
import numpy as np
import logging
from pyCGM2 import enums
from pyCGM2.Model import modelFilters, modelDecorator


class argsManager_cgm(object):
    def __init__(self, settings, args):
        self.settings = settings
        self.args = args


    def getHeadFlat(self):
        if self.args.headFlat is not None:
            logging.warning("Head flat option : %s"%(str(bool(self.args.headFlat))))
            return  bool(self.args.headFlat)
        else:
            return bool(self.settings["Calibration"]["Head flat"])



    def getLeftFlatFoot(self):
        if self.args.leftFlatFoot is not None:
            logging.warning("Left flat foot forces : %s"%(str(bool(self.args.leftFlatFoot))))
            return  bool(self.args.leftFlatFoot)
        else:
            return bool(self.settings["Calibration"]["Left flat foot"])

    def getRightFlatFoot(self):
        if self.args.rightFlatFoot is not None:
            logging.warning("Right flat foot forces : %s"%(str(bool(self.args.rightFlatFoot))))
            return bool(self.args.rightFlatFoot)
        else:
            return  bool(self.settings["Calibration"]["Right flat foot"])


    def getMarkerDiameter(self):
        if self.args.markerDiameter is not None:
            logging.warning("marker diameter forced : %s", str(float(self.args.markerDiameter)))
            return float(self.args.markerDiameter)
        else:
            return float(self.settings["Global"]["Marker diameter"])

    def getPointSuffix(self,checkValue):

        if hasattr(self.args,"check") and self.args.check:
            return checkValue
        else:
            if self.args.pointSuffix is not None:
                return self.args.pointSuffix
            else:
                if self.settings["Global"]["Point suffix"] =="None":
                    self.settings["Global"]["Point suffix"] = None

                return self.settings["Global"]["Point suffix"]

    def enableIKflag(self):
        if self.args.noIk:
            return False
        else:
            if self.settings["Global"]["EnableIK"]:
                return True
            else:
                return False

    def getMomentProjection(self):
        if self.args.proj is not None:
            if self.args.proj == "Distal":
                return  enums.MomentProjection.Distal
            elif self.args.proj == "Proximal":
                return  enums.MomentProjection.Proximal
            elif self.args.proj == "Global":
                return  enums.MomentProjection.Global
            elif self.args.proj == "JCS":
                return enums.MomentProjection.JCS
            else:
                raise Exception("[pyCGM2] Moment projection doesn t recognise in your settings. choice is Proximal, Distal or Global")

        else:
            if self.settings["Fitting"]["Moment Projection"] == "Distal":
                return  enums.MomentProjection.Distal
            elif self.settings["Fitting"]["Moment Projection"] == "Proximal":
                return  enums.MomentProjection.Proximal
            elif self.settings["Fitting"]["Moment Projection"] == "Global":
                return  enums.MomentProjection.Global
            elif self.settings["Fitting"]["Moment Projection"] == "JCS":
                return enums.MomentProjection.JCS

            else:
                raise Exception("[pyCGM2] Moment projection doesn t recognise in your settings. choice is Proximal, Distal or Global")

    def getManualForcePlateAssign(self):
        return self.args.mfpa

    def getIkWeightFile(self):
        return self.args.ikWeightFile

    def forceHjc(self,side):
        if side == "left":
            if self.args.forceLHJC is not None:
                if len(self.args.forceLHJC) == 3:
                    lhjc = [float(i) for i in self.args.forceLHJC]
                else:
                    raise Exception("[pyCGM2] : left hjc position must have 3 components")
                return lhjc
            else:
                return None

        if side == "right":
            if self.args.forceRHJC is not None:
                if len(self.args.forceRHJC) == 3:
                    rhjc = [float(i) for i in self.args.forceRHJC]
                else:
                    raise Exception("[pyCGM2] : right hjc position must have 3 components")
                return rhjc
            else:
                return None

    def getAnalysisTitle(self):
        return self.args.analysisTitle


class argsManager_cgm1(argsManager_cgm):
    def __init__(self, settings, args):
        super(argsManager_cgm1, self).__init__(settings, args)

    def getMomentProjection(self):
        if self.args.proj is not None:
            if self.args.proj == "Distal":
                return  enums.MomentProjection.Distal
            elif self.args.proj == "Proximal":
                return  enums.MomentProjection.Proximal
            elif self.args.proj == "Global":
                return  enums.MomentProjection.Global
            else:
                raise Exception("[pyCGM2] Moment projection doesn t recognise in your settings. choice is Proximal, Distal or Global")

        else:
            if self.settings["Fitting"]["Moment Projection"] == "Distal":
                return  enums.MomentProjection.Distal
            elif self.settings["Fitting"]["Moment Projection"] == "Proximal":
                return  enums.MomentProjection.Proximal
            elif self.settings["Fitting"]["Moment Projection"] == "Global":
                return  enums.MomentProjection.Global

            else:
                raise Exception("[pyCGM2] Moment projection doesn t recognise in your settings. choice is Proximal, Distal or Global")



# -*- coding: utf-8 -*-
"""Nexus Operation : **CGM2.1 Fitting**

:param --proj [string]: define in which coordinate system joint moment will be expressed (Choice : Distal, Proximal, Global)
:param -md, --markerDiameter [int]: marker diameter
:param -ps, --pointSuffix [string]: suffix adds to the vicon nomenclature outputs
:param --check [bool]: add "cgm2.1" as point suffix


Examples:
    In the script argument box of a python nexus operation, you can edit:

    >>> --proj=Global
    (means joint moments will be expressed into the Global Coordinate system)

"""
import os
import logging
import argparse

# pyCGM2 settings
import pyCGM2
from pyCGM2 import log; log.setLoggingLevel(logging.INFO)

# vicon nexus
try:
    import ViconNexus
except:
    from viconnexusapi import ViconNexus

# pyCGM2 libraries
from pyCGM2.Configurator import CgmArgsManager
from pyCGM2.Lib.CGM import  cgm2_1
from pyCGM2.Utils import files
from pyCGM2.Nexus import nexusFilters,nexusTools,nexusUtils,nexusUtils


def main():

    parser = argparse.ArgumentParser(description='CGM2-1 Fitting')
    parser.add_argument('--proj', type=str, help='Moment Projection. Choice : Distal, Proximal, Global')
    parser.add_argument('-md','--markerDiameter', type=float, help='marker diameter')
    parser.add_argument('-ps','--pointSuffix', type=str, help='suffix of model outputs')
    parser.add_argument('--check', action='store_true', help='force model output suffix')
    args = parser.parse_args()


    NEXUS = ViconNexus.ViconNexus()
    NEXUS_PYTHON_CONNECTED = NEXUS.Client.IsConnected()



    if NEXUS_PYTHON_CONNECTED: # run Operation

        # --------------------------GLOBAL SETTINGS ------------------------------------
        # global setting ( in user/AppData)
        if os.path.isfile(pyCGM2.PYCGM2_APPDATA_PATH + "CGM2_1-pyCGM2.settings"):
            settings = files.openFile(pyCGM2.PYCGM2_APPDATA_PATH,"CGM2_1-pyCGM2.settings")
        else:
            settings = files.openFile(pyCGM2.PYCGM2_SETTINGS_FOLDER,"CGM2_1-pyCGM2.settings")


        # --------------------------CONFIG ------------------------------------
        argsManager = CgmArgsManager.argsManager_cgm(settings,args)
        markerDiameter = argsManager.getMarkerDiameter()
        pointSuffix = argsManager.getPointSuffix("cgm2.1")
        momentProjection =  argsManager.getMomentProjection()

        # --------------------------LOADING ------------------------------------
        DATA_PATH, reconstructFilenameLabelledNoExt = NEXUS.GetTrialName()

        reconstructFilenameLabelled = reconstructFilenameLabelledNoExt+".c3d"

        logging.info( "data Path: "+ DATA_PATH )
        logging.info( "calibration file: "+ reconstructFilenameLabelled)


        # --------------------------SUBJECT ------------------------------------
        subjects = NEXUS.GetSubjectNames()
        subject = nexusTools.getActiveSubject(NEXUS)
        logging.info(  "Subject name : " + subject  )

        # --------------------pyCGM2 MODEL ------------------------------
        model = files.loadModel(DATA_PATH,subject)

        # -------------------------- MP ------------------------------------
        # allow alteration of thigh offset
        model.mp_computed["LeftThighRotationOffset"] =   NEXUS.GetSubjectParamDetails( subject, "LeftThighRotation")[0]
        model.mp_computed["RightThighRotationOffset"] =   NEXUS.GetSubjectParamDetails( subject, "RightThighRotation")[0]



        # check model
        logging.info("loaded model : %s" %(model.version ))
        if model.version != "CGM2.1":
            raise Exception ("%s-pyCGM2.model file was not calibrated from the CGM2.1 calibration pipeline"%subject)

        # --------------------------SESSION INFOS ------------------------------------

        #  translators management
        translators = files.getTranslators(DATA_PATH,"CGM2_1.translators")
        if not translators:  translators = settings["Translators"]

        #force plate assignement from Nexus
        mfpa = nexusTools.getForcePlateAssignment(NEXUS)

        # btkAcq builder
        nacf = nexusFilters.NexusConstructAcquisitionFilter(DATA_PATH,reconstructFilenameLabelledNoExt,subject)
        acq = nacf.build()

        # --------------------------MODELLING PROCESSING -----------------------
        acqGait = cgm2_1.fitting(model,DATA_PATH, reconstructFilenameLabelled,
            translators,
            markerDiameter,
            pointSuffix,
            mfpa,momentProjection,
            forceBtkAcq=acq)

        # ----------------------DISPLAY ON VICON-------------------------------
        nexusFilters.NexusModelFilter(NEXUS,model,acqGait,subject,pointSuffix).run()
        nexusTools.createGeneralEvents(NEXUS,subject,acqGait,["Left-FP","Right-FP"])

        # ========END of the nexus OPERATION if run from Nexus  =========


    else:
        raise Exception("NO Nexus connection. Turn on Nexus")

if __name__ == "__main__":

    main()



# -*- coding: utf-8 -*-
#import ipdb
import logging
import matplotlib.pyplot as plt
import argparse


# pyCGM2 settings
import pyCGM2

# pyCGM2 libraries
from pyCGM2.Tools import btkTools
from pyCGM2 import enums

from pyCGM2.Model import modelFilters, modelDecorator,bodySegmentParameters
from pyCGM2.Model.CGM2 import cgm,cgm2
from pyCGM2.Model.CGM2 import decorators
from pyCGM2.ForcePlates import forceplates
from pyCGM2.Model.Opensim import opensimFilters
from pyCGM2.Processing import progressionFrame
from pyCGM2.Signal import signal_processing


def calibrate(DATA_PATH,calibrateFilenameLabelled,translators,weights,
              required_mp,optional_mp,
              ik_flag,leftFlatFoot,rightFlatFoot,headFlat,
              markerDiameter,hjcMethod,
              pointSuffix,**kwargs):
    """
    Calibration of the CGM2.3

    :param DATA_PATH [str]: path to your data
    :param calibrateFilenameLabelled [str]: c3d file
    :param translators [dict]:  translators to apply
    :param required_mp [dict]: required anthropometric data
    :param optional_mp [dict]: optional anthropometric data (ex: LThighOffset,...)
    :param ik_flag [bool]: enable the inverse kinematic solver
    :param leftFlatFoot [bool]: enable of the flat foot option for the left foot
    :param rightFlatFoot [bool]: enable of the flat foot option for the right foot
    :param headFlat [bool]: enable of the head flat  option
    :param markerDiameter [double]: marker diameter (mm)
    :param hjcMethod [str or list of 3 float]: method for locating the hip joint centre
    :param pointSuffix [str]: suffix to add to model outputs

    """
    # --------------------------STATIC FILE WITH TRANSLATORS --------------------------------------

    if "Fitting" in weights.keys():
        weights  = weights["Fitting"]["Weight"]


    # --- btk acquisition ----
    if "forceBtkAcq" in kwargs.keys():
        acqStatic = kwargs["forceBtkAcq"]
    else:
        acqStatic = btkTools.smartReader((DATA_PATH+calibrateFilenameLabelled))

    btkTools.checkMultipleSubject(acqStatic)
    if btkTools.isPointExist(acqStatic,"SACR"):
        translators["LPSI"] = "SACR"
        translators["RPSI"] = "SACR"
        logging.info("[pyCGM2] Sacrum marker detected")

    acqStatic =  btkTools.applyTranslators(acqStatic,translators)

    # ---check marker set used----
    dcm = cgm.CGM.detectCalibrationMethods(acqStatic)

    # --------------------------MODEL--------------------------------------
    # ---definition---
    model=cgm2.CGM2_3()
    model.configure(acq=acqStatic,detectedCalibrationMethods=dcm)
    model.addAnthropoInputParameters(required_mp,optional=optional_mp)

    # --store calibration parameters--
    model.setStaticFilename(calibrateFilenameLabelled)
    model.setCalibrationProperty("leftFlatFoot",leftFlatFoot)
    model.setCalibrationProperty("rightFlatFoot",rightFlatFoot)
    model.setCalibrationProperty("headFlat",headFlat)
    model.setCalibrationProperty("markerDiameter",markerDiameter)




    # --------------------------STATIC CALBRATION--------------------------
    scp=modelFilters.StaticCalibrationProcedure(model) # load calibration procedure

    # ---initial calibration filter----
    # use if all optional mp are zero
    modelFilters.ModelCalibrationFilter(scp,acqStatic,model,
                                        leftFlatFoot = leftFlatFoot, rightFlatFoot = rightFlatFoot,
                                        headFlat= headFlat,
                                        markerDiameter=markerDiameter,
                                        ).compute()

    # ---- Decorators -----
    decorators.applyBasicDecorators(dcm, model,acqStatic,optional_mp,markerDiameter)
    decorators.applyHJCDecorators(model,hjcMethod)


    # ----Final Calibration filter if model previously decorated -----
    if model.decoratedModel:
        # initial static filter
        modelFilters.ModelCalibrationFilter(scp,acqStatic,model,
                           leftFlatFoot = leftFlatFoot, rightFlatFoot = rightFlatFoot,
                           headFlat= headFlat,
                           markerDiameter=markerDiameter).compute()


    # ----------------------CGM MODELLING----------------------------------
    # ----motion filter----
    modMotion=modelFilters.ModelMotionFilter(scp,acqStatic,model,enums.motionMethod.Sodervisk,
                                              markerDiameter=markerDiameter)

    modMotion.compute()

    if "noKinematicsCalculation" in kwargs.keys() and kwargs["noKinematicsCalculation"]:
        logging.warning("[pyCGM2] No Kinematic calculation done for the static file")
        return model, acqStatic
    else:
        if model.getBodyPart() == enums.BodyPart.UpperLimb:
            ik_flag = False
            logging.warning("[pyCGM2] Fitting only applied for the upper limb")

        if ik_flag:
            #                        ---OPENSIM IK---

            # --- opensim calibration Filter ---
            osimfile = pyCGM2.OPENSIM_PREBUILD_MODEL_PATH + "models\\osim\\lowerLimb_ballsJoints.osim"    # osimfile
            markersetFile = pyCGM2.OPENSIM_PREBUILD_MODEL_PATH + "models\\settings\\cgm2_3\\cgm2_3-markerset.xml" # markerset
            cgmCalibrationprocedure = opensimFilters.CgmOpensimCalibrationProcedures(model) # procedure

            oscf = opensimFilters.opensimCalibrationFilter(osimfile,
                                                    model,
                                                    cgmCalibrationprocedure,
                                                    (DATA_PATH))
            oscf.addMarkerSet(markersetFile)
            scalingOsim = oscf.build()


            # --- opensim Fitting Filter ---
            iksetupFile = pyCGM2.OPENSIM_PREBUILD_MODEL_PATH + "models\\settings\\cgm2_3\\cgm2_3-ikSetUp_template.xml" # ik tool file

            cgmFittingProcedure = opensimFilters.CgmOpensimFittingProcedure(model) # procedure
            cgmFittingProcedure.updateMarkerWeight("LASI",weights["LASI"])
            cgmFittingProcedure.updateMarkerWeight("RASI",weights["RASI"])
            cgmFittingProcedure.updateMarkerWeight("LPSI",weights["LPSI"])
            cgmFittingProcedure.updateMarkerWeight("RPSI",weights["RPSI"])
            cgmFittingProcedure.updateMarkerWeight("RTHI",weights["RTHI"])
            cgmFittingProcedure.updateMarkerWeight("RKNE",weights["RKNE"])
            cgmFittingProcedure.updateMarkerWeight("RTIB",weights["RTIB"])
            cgmFittingProcedure.updateMarkerWeight("RANK",weights["RANK"])
            cgmFittingProcedure.updateMarkerWeight("RHEE",weights["RHEE"])
            cgmFittingProcedure.updateMarkerWeight("RTOE",weights["RTOE"])
            cgmFittingProcedure.updateMarkerWeight("LTHI",weights["LTHI"])
            cgmFittingProcedure.updateMarkerWeight("LKNE",weights["LKNE"])
            cgmFittingProcedure.updateMarkerWeight("LTIB",weights["LTIB"])
            cgmFittingProcedure.updateMarkerWeight("LANK",weights["LANK"])
            cgmFittingProcedure.updateMarkerWeight("LHEE",weights["LHEE"])
            cgmFittingProcedure.updateMarkerWeight("LTOE",weights["LTOE"])

            cgmFittingProcedure.updateMarkerWeight("LTHAP",weights["LTHAP"])
            cgmFittingProcedure.updateMarkerWeight("LTHAD",weights["LTHAD"])
            cgmFittingProcedure.updateMarkerWeight("LTIAP",weights["LTIAP"])
            cgmFittingProcedure.updateMarkerWeight("LTIAD",weights["LTIAD"])
            cgmFittingProcedure.updateMarkerWeight("RTHAP",weights["RTHAP"])
            cgmFittingProcedure.updateMarkerWeight("RTHAD",weights["RTHAD"])
            cgmFittingProcedure.updateMarkerWeight("RTIAP",weights["RTIAP"])
            cgmFittingProcedure.updateMarkerWeight("RTIAD",weights["RTIAD"])

            osrf = opensimFilters.opensimFittingFilter(iksetupFile,
                                                              scalingOsim,
                                                              cgmFittingProcedure,
                                                              (DATA_PATH) )
            acqStaticIK = osrf.run(acqStatic,(DATA_PATH + calibrateFilenameLabelled ))



        # eventual static acquisition to consider for joint kinematics
        finalAcqStatic = acqStaticIK if ik_flag else acqStatic

        # --- final pyCGM2 model motion Filter ---
        # use fitted markers
        modMotionFitted=modelFilters.ModelMotionFilter(scp,finalAcqStatic,model,enums.motionMethod.Sodervisk)
        modMotionFitted.compute()

        if "displayCoordinateSystem" in kwargs.keys() and kwargs["displayCoordinateSystem"]:
            csp = modelFilters.ModelCoordinateSystemProcedure(model)
            csdf = modelFilters.CoordinateSystemDisplayFilter(csp,model,finalAcqStatic)
            csdf.setStatic(False)
            csdf.display()

        #---- Joint kinematics----
        # relative angles
        modelFilters.ModelJCSFilter(model,finalAcqStatic).compute(description="vectoriel", pointLabelSuffix=pointSuffix)

        # detection of traveling axis + absolute angle
        if model.m_bodypart != enums.BodyPart.UpperLimb:
            pfp = progressionFrame.PelvisProgressionFrameProcedure()
        else:
            pfp = progressionFrame.ThoraxProgressionFrameProcedure()

        pff = progressionFrame.ProgressionFrameFilter(finalAcqStatic,pfp)
        pff.compute()
        globalFrame = pff.outputs["globalFrame"]
        forwardProgression = pff.outputs["forwardProgression"]


        if model.m_bodypart != enums.BodyPart.UpperLimb:
                modelFilters.ModelAbsoluteAnglesFilter(model,finalAcqStatic,
                                                       segmentLabels=["Left Foot","Right Foot","Pelvis"],
                                                        angleLabels=["LFootProgress", "RFootProgress","Pelvis"],
                                                        eulerSequences=["TOR","TOR", "ROT"],
                                                        globalFrameOrientation = globalFrame,
                                                        forwardProgression = forwardProgression).compute(pointLabelSuffix=pointSuffix)

        if model.m_bodypart == enums.BodyPart.LowerLimbTrunk:
                modelFilters.ModelAbsoluteAnglesFilter(model,finalAcqStatic,
                                              segmentLabels=["Thorax"],
                                              angleLabels=["Thorax"],
                                              eulerSequences=["YXZ"],
                                              globalFrameOrientation = globalFrame,
                                              forwardProgression = forwardProgression).compute(pointLabelSuffix=pointSuffix)

        if model.m_bodypart == enums.BodyPart.UpperLimb or model.m_bodypart == enums.BodyPart.FullBody:

                modelFilters.ModelAbsoluteAnglesFilter(model,finalAcqStatic,
                                              segmentLabels=["Thorax","Head"],
                                              angleLabels=["Thorax", "Head"],
                                              eulerSequences=["YXZ","TOR"],
                                              globalFrameOrientation = globalFrame,
                                              forwardProgression = forwardProgression).compute(pointLabelSuffix=pointSuffix)
        # BSP model
        bspModel = bodySegmentParameters.Bsp(model)
        bspModel.compute()

        if  model.m_bodypart == enums.BodyPart.FullBody:
            modelFilters.CentreOfMassFilter(model,finalAcqStatic).compute(pointLabelSuffix=pointSuffix)



        return model, finalAcqStatic


def fitting(model,DATA_PATH, reconstructFilenameLabelled,
    translators,weights,
    ik_flag,markerDiameter,
    pointSuffix,
    mfpa,
    momentProjection,**kwargs):

    """
    Fitting of the CGM2.3

    :param model [str]: pyCGM2 model previously calibrated
    :param DATA_PATH [str]: path to your data
    :param reconstructFilenameLabelled [string list]: c3d files
    :param translators [dict]:  translators to apply
    :param ik_flag [bool]: enable the inverse kinematic solver
    :param mfpa [str]: manual force plate assignement
    :param markerDiameter [double]: marker diameter (mm)
    :param pointSuffix [str]: suffix to add to model outputs
    :param momentProjection [str]: Coordinate system in which joint moment is expressed
    """

    if "Fitting" in weights.keys():
        weights  = weights["Fitting"]["Weight"]

    # --------------------------ACQ WITH TRANSLATORS --------------------------------------

    # --- btk acquisition ----
    if "forceBtkAcq" in kwargs.keys():
        acqGait = kwargs["forceBtkAcq"]
    else:
        acqGait = btkTools.smartReader((DATA_PATH + reconstructFilenameLabelled))

    btkTools.checkMultipleSubject(acqGait)
    if btkTools.isPointExist(acqGait,"SACR"):
        translators["LPSI"] = "SACR"
        translators["RPSI"] = "SACR"
        logging.info("[pyCGM2] Sacrum marker detected")

    acqGait =  btkTools.applyTranslators(acqGait,translators)
    trackingMarkers = model.getTrackingMarkers(acqGait)
    validFrames,vff,vlf = btkTools.findValidFrames(acqGait,trackingMarkers)
    # filtering
    # -----------------------
    if "fc_lowPass_marker" in kwargs.keys() and kwargs["fc_lowPass_marker"]!=0 :
        fc = kwargs["fc_lowPass_marker"]
        order = 4
        if "order_lowPass_marker" in kwargs.keys():
            order = kwargs["order_lowPass_marker"]
        signal_processing.markerFiltering(acqGait,trackingMarkers,order=order, fc =fc)

    if "fc_lowPass_forcePlate" in kwargs.keys() and kwargs["fc_lowPass_forcePlate"]!=0 :
        fc = kwargs["fc_lowPass_forcePlate"]
        order = 4
        if "order_lowPass_forcePlate" in kwargs.keys():
            order = kwargs["order_lowPass_forcePlate"]
        signal_processing.forcePlateFiltering(acqGait,order=order, fc =fc)


    # --- initial motion Filter ---
    scp=modelFilters.StaticCalibrationProcedure(model)
    # section to remove : - copy motion of ProximalShank from Shank with Sodervisk
    modMotion=modelFilters.ModelMotionFilter(scp,acqGait,model,enums.motionMethod.Sodervisk)
    modMotion.compute()
    # /section to remove


    if model.getBodyPart() == enums.BodyPart.UpperLimb:
        ik_flag = False
        logging.warning("[pyCGM2] Fitting only applied for the upper limb")

    if ik_flag:

        #                        ---OPENSIM IK---

        # --- opensim calibration Filter ---
        osimfile = pyCGM2.OPENSIM_PREBUILD_MODEL_PATH + "models\\osim\\lowerLimb_ballsJoints.osim"    # osimfile
        markersetFile = pyCGM2.OPENSIM_PREBUILD_MODEL_PATH + "models\\settings\\cgm2_3\\cgm2_3-markerset.xml" # markerset
        cgmCalibrationprocedure = opensimFilters.CgmOpensimCalibrationProcedures(model) # procedure

        oscf = opensimFilters.opensimCalibrationFilter(osimfile,
                                                model,
                                                cgmCalibrationprocedure,
                                                (DATA_PATH))
        oscf.addMarkerSet(markersetFile)
        scalingOsim = oscf.build()


        # --- opensim Fitting Filter ---
        iksetupFile = pyCGM2.OPENSIM_PREBUILD_MODEL_PATH + "models\\settings\\cgm2_3\\cgm2_3-ikSetUp_template.xml" # ik tl file

        cgmFittingProcedure = opensimFilters.CgmOpensimFittingProcedure(model) # procedure
        cgmFittingProcedure.updateMarkerWeight("LASI",weights["LASI"])
        cgmFittingProcedure.updateMarkerWeight("RASI",weights["RASI"])
        cgmFittingProcedure.updateMarkerWeight("LPSI",weights["LPSI"])
        cgmFittingProcedure.updateMarkerWeight("RPSI",weights["RPSI"])
        cgmFittingProcedure.updateMarkerWeight("RTHI",weights["RTHI"])
        cgmFittingProcedure.updateMarkerWeight("RKNE",weights["RKNE"])
        cgmFittingProcedure.updateMarkerWeight("RTIB",weights["RTIB"])
        cgmFittingProcedure.updateMarkerWeight("RANK",weights["RANK"])
        cgmFittingProcedure.updateMarkerWeight("RHEE",weights["RHEE"])
        cgmFittingProcedure.updateMarkerWeight("RTOE",weights["RTOE"])
        cgmFittingProcedure.updateMarkerWeight("LTHI",weights["LTHI"])
        cgmFittingProcedure.updateMarkerWeight("LKNE",weights["LKNE"])
        cgmFittingProcedure.updateMarkerWeight("LTIB",weights["LTIB"])
        cgmFittingProcedure.updateMarkerWeight("LANK",weights["LANK"])
        cgmFittingProcedure.updateMarkerWeight("LHEE",weights["LHEE"])
        cgmFittingProcedure.updateMarkerWeight("LTOE",weights["LTOE"])

        cgmFittingProcedure.updateMarkerWeight("LTHAP",weights["LTHAP"])
        cgmFittingProcedure.updateMarkerWeight("LTHAD",weights["LTHAD"])
        cgmFittingProcedure.updateMarkerWeight("LTIAP",weights["LTIAP"])
        cgmFittingProcedure.updateMarkerWeight("LTIAD",weights["LTIAD"])
        cgmFittingProcedure.updateMarkerWeight("RTHAP",weights["RTHAP"])
        cgmFittingProcedure.updateMarkerWeight("RTHAD",weights["RTHAD"])
        cgmFittingProcedure.updateMarkerWeight("RTIAP",weights["RTIAP"])
        cgmFittingProcedure.updateMarkerWeight("RTIAD",weights["RTIAD"])

        osrf = opensimFilters.opensimFittingFilter(iksetupFile,
                                                          scalingOsim,
                                                          cgmFittingProcedure,
                                                          (DATA_PATH) )

        logging.info("-------INVERSE KINEMATICS IN PROGRESS----------")
        acqIK = osrf.run(acqGait,(DATA_PATH + reconstructFilenameLabelled ))
        logging.info("-------INVERSE KINEMATICS DONE-----------------")

    # eventual gait acquisition to consider for joint kinematics
    finalAcqGait = acqIK if ik_flag else acqGait

    # --- final pyCGM2 model motion Filter ---
    # use fitted markers
    modMotionFitted=modelFilters.ModelMotionFilter(scp,finalAcqGait,model,enums.motionMethod.Sodervisk ,
                                              markerDiameter=markerDiameter)

    modMotionFitted.compute()

    if "displayCoordinateSystem" in kwargs.keys() and kwargs["displayCoordinateSystem"]:
        csp = modelFilters.ModelCoordinateSystemProcedure(model)
        csdf = modelFilters.CoordinateSystemDisplayFilter(csp,model,finalAcqGait)
        csdf.setStatic(False)
        csdf.display()

    #---- Joint kinematics----
    # relative angles
    modelFilters.ModelJCSFilter(model,finalAcqGait).compute(description="vectoriel", pointLabelSuffix=pointSuffix)

    # detection of traveling axis + absolute angle
    if model.m_bodypart != enums.BodyPart.UpperLimb:
        pfp = progressionFrame.PelvisProgressionFrameProcedure()
    else:
        pfp = progressionFrame.ThoraxProgressionFrameProcedure()

    pff = progressionFrame.ProgressionFrameFilter(finalAcqGait,pfp)
    pff.compute()
    globalFrame = pff.outputs["globalFrame"]
    forwardProgression = pff.outputs["forwardProgression"]

    if model.m_bodypart != enums.BodyPart.UpperLimb:
            modelFilters.ModelAbsoluteAnglesFilter(model,finalAcqGait,
                                                   segmentLabels=["Left Foot","Right Foot","Pelvis"],
                                                    angleLabels=["LFootProgress", "RFootProgress","Pelvis"],
                                                    eulerSequences=["TOR","TOR", "ROT"],
                                                    globalFrameOrientation = globalFrame,
                                                    forwardProgression = forwardProgression).compute(pointLabelSuffix=pointSuffix)

    if model.m_bodypart == enums.BodyPart.LowerLimbTrunk:
            modelFilters.ModelAbsoluteAnglesFilter(model,finalAcqGait,
                                          segmentLabels=["Thorax"],
                                          angleLabels=["Thorax"],
                                          eulerSequences=["YXZ"],
                                          globalFrameOrientation = globalFrame,
                                          forwardProgression = forwardProgression).compute(pointLabelSuffix=pointSuffix)

    if model.m_bodypart == enums.BodyPart.UpperLimb or model.m_bodypart == enums.BodyPart.FullBody:

            modelFilters.ModelAbsoluteAnglesFilter(model,finalAcqGait,
                                          segmentLabels=["Thorax","Head"],
                                          angleLabels=["Thorax", "Head"],
                                          eulerSequences=["YXZ","TOR"],
                                          globalFrameOrientation = globalFrame,
                                          forwardProgression = forwardProgression).compute(pointLabelSuffix=pointSuffix)

    #---- Body segment parameters----
    bspModel = bodySegmentParameters.Bsp(model)
    bspModel.compute()

    if  model.m_bodypart == enums.BodyPart.FullBody:
        modelFilters.CentreOfMassFilter(model,finalAcqGait).compute(pointLabelSuffix=pointSuffix)

    # Inverse dynamics
    if btkTools.checkForcePlateExist(acqGait):
        if model.m_bodypart != enums.BodyPart.UpperLimb:
            # --- force plate handling----
            # find foot  in contact
            mappedForcePlate = forceplates.matchingFootSideOnForceplate(finalAcqGait,mfpa=mfpa)
            forceplates.addForcePlateGeneralEvents(finalAcqGait,mappedForcePlate)
            logging.warning("Manual Force plate assignment : %s" %mappedForcePlate)


            # assembly foot and force plate
            modelFilters.ForcePlateAssemblyFilter(model,finalAcqGait,mappedForcePlate,
                                     leftSegmentLabel="Left Foot",
                                     rightSegmentLabel="Right Foot").compute(pointLabelSuffix=pointSuffix)

            #---- Joint kinetics----
            idp = modelFilters.CGMLowerlimbInverseDynamicProcedure()
            modelFilters.InverseDynamicFilter(model,
                                 finalAcqGait,
                                 procedure = idp,
                                 projection = momentProjection,
                                 globalFrameOrientation = globalFrame,
                                 forwardProgression = forwardProgression
                                 ).compute(pointLabelSuffix=pointSuffix)


            #---- Joint energetics----
            modelFilters.JointPowerFilter(model,finalAcqGait).compute(pointLabelSuffix=pointSuffix)

    #---- zero unvalid frames ---
    btkTools.applyValidFramesOnOutput(finalAcqGait,validFrames)


    return finalAcqGait



# -*- coding: utf-8 -*-
"""Nexus Operation : **plotTemporalEmg**

The script displays rectified EMG with time as x-axis

:param -fso, --footStrikeOffset [int]: add an offset on all foot strike events
:param -foo, --footOffOffset [int]: add an offset on all foot off events


Examples:
    In the script argument box of a python nexus operation, you can edit:

    >>>  -fso=10
    (add 10 frames to all foot strike events)


"""

import logging
import argparse

# pyCGM2 settings
import pyCGM2
from pyCGM2 import log; log.setLoggingLevel(logging.INFO)

# vicon nexus
try:
    import ViconNexus
except:
    from viconnexusapi import ViconNexus


# pyCGM2 libraries

from pyCGM2.Events import events
from pyCGM2.Nexus import nexusTools,nexusFilters

def main():

    parser = argparse.ArgumentParser(description='ZeniDetector')
    parser.add_argument('-fso','--footStrikeOffset', type=int, help='systenatic foot strike offset on both side')
    parser.add_argument('-foo','--footOffOffset', type=int, help='systenatic foot off offset on both side')
    args = parser.parse_args()


    NEXUS = ViconNexus.ViconNexus()
    NEXUS_PYTHON_CONNECTED = NEXUS.Client.IsConnected()


    if NEXUS_PYTHON_CONNECTED: # run Operation

        # ----------------------INPUTS-------------------------------------------
        # --- acquisition file and path----
        DATA_PATH, reconstructFilenameLabelledNoExt = NEXUS.GetTrialName()

        reconstructFilenameLabelled = reconstructFilenameLabelledNoExt+".c3d"

        logging.info( "data Path: "+ DATA_PATH )
        logging.info( "calibration file: "+ reconstructFilenameLabelled)

        #acqGait = btkTools.smartReader(str(DATA_PATH + reconstructFilenameLabelled))

        # --------------------------SUBJECT -----------------------------------

        # Notice : Work with ONE subject by session
        subjects = NEXUS.GetSubjectNames()
        subject = nexusTools.getActiveSubject(NEXUS)
        logging.info(  "Subject name : " + subject  )

        # --- btk acquisition ----
        nacf = nexusFilters.NexusConstructAcquisitionFilter(DATA_PATH,reconstructFilenameLabelledNoExt,subject)
        acqGait = nacf.build()


        # ----------------------EVENT DETECTOR-------------------------------
        evp = events.ZeniProcedure()
        if args.footStrikeOffset is not None:
            evp.setFootStrikeOffset(args.footStrikeOffset)
        if args.footOffOffset is not None:
            evp.setFootOffOffset(args.footOffOffset)


        evf = events.EventFilter(evp,acqGait)
        evf.detect()
        state = evf.getState()

        # ----------------------DISPLAY ON VICON-------------------------------
        nexusTools.createEvents(NEXUS,subject,acqGait,["Foot Strike","Foot Off"])
        # ========END of the nexus OPERATION if run from Nexus  =========



    else:
        raise Exception("NO Nexus connection. Turn on Nexus")

if __name__ == "__main__":


    main()
